require 'osx/cocoa'
include OSX
OSX.require_framework 'AVFoundation'
require 'pp'
require 'rational'
require 'time'
require 'erb'
require 'rubygems'
require 'nokogiri'
gem 'tzinfo'
require 'tzinfo' # used for server time zone
#require 'titlecase' # e.g. "Bob's big boy".titlecase vs "Bob's big boy".capitalize
bundle = "#{File.dirname(__FILE__)}/lumberyard_methods.bundle"
if File.exist?(bundle)
	require 'lumberyard_methods.bundle'
else
	p "NO BUNDLE #{bundle} !!!\n"
end

class String
	def hasPrefix?(str)
		prefix = self[0 .. str.length-1]
		if prefix==str
			return true
		else
			return false
		end
	end
	
	def hasSuffix?(str)
		if File.extname(self)==str
			return true
		else
			return false
		end
	end
end

class AppController < OSX::NSObject
	ib_outlets :debugMenu, :localhostMenuItem, :textmateMenuItem
	ib_outlets :textFieldCell, :checkBoxCell
	ib_outlets :mainWindow, :loginButton, :importXMLButton, :xmlName, :eventPopup, :transcriptsButton, :cardPopup, :posNegPopup, :dayPopup, :hrsPopup, :minPopup, :secPopup, :keywordsTableView, :mediaTableView, :peopleActivitiesCheckbox, :peopleKeywordsCheckbox, :peopleTranscriptsCheckbox, :activitesCheckbox, :keywordsCheckbox, :favoritesCheckbox, :titlesCheckbox, :multicamRadioButtons, :sendToNLEButton, :sendToBuilderButton, :spinner
	ib_outlets :progressPanel, :progressView, :progressIndicator, :progressMessage
	ib_outlets :preferencesWindow, :preferencesTabs, :usernameField, :passwordField, :contentCreatedRadioButtons, :keywordOptionsPopup, :debugMenuCheckbox, :tzCountryPopup, :tzCityPopup

	def initialize
		default_values = { "ContentCreated" => 0, "Localhost" => "0", "TextMate" => "0", "KeywordOptions" => "Keywords", "Debug" => false }
		defaultsController = NSUserDefaultsController.sharedUserDefaultsController
		defaultsController.setInitialValues(default_values)
	end
	
	def awakeFromNib
		puts("awakeFromNib")
		@keywordORmarker = "keyword"
		@xmlName.setStringValue("")
		@dayPopup.removeAllItems
		@hrsPopup.removeAllItems
		@minPopup.removeAllItems
		@secPopup.removeAllItems
		0.upto(365) do |i|
			@dayPopup.addItemWithTitle(i.to_s)
			if i<24
				@hrsPopup.addItemWithTitle("%02d" % i)
			end
			if i<60
				@minPopup.addItemWithTitle("%02d" % i)
				@secPopup.addItemWithTitle("%02d" % i)
			end
		end
		@tzCountryPopup.removeAllItems
		@tzCountryPopup.addItemWithTitle("United States")
		@tzCityPopup.removeAllItems
		@tzCityPopup.addItemWithTitle("America - Los Angeles")
		@peopleActivitiesCheckbox.setState(0)
		@peopleActivitiesCheckbox.setEnabled(false)
		@peopleKeywordsCheckbox.setState(0)
		@peopleKeywordsCheckbox.setEnabled(false)
		@peopleTranscriptsCheckbox.setState(0)
		@peopleTranscriptsCheckbox.setEnabled(false)
		@activitesCheckbox.setState(0)
		@activitesCheckbox.setEnabled(false)
		@keywordsCheckbox.setState(0)
		@keywordsCheckbox.setEnabled(false)
		@favoritesCheckbox.setState(0)
		@favoritesCheckbox.setEnabled(false)
		@titlesCheckbox.setState(0)
		@titlesCheckbox.setEnabled(false)
		@multicamRadioButtons.setEnabled(false)
		@multicamRadioButtons.selectCellWithTag(0)
		unlockInterface(false)
		#check for Application Support subfolder
		paths = NSFileManager.defaultManager.URLsForDirectory_inDomains(NSApplicationSupportDirectory, NSUserDomainMask)
		if paths.length > 0
			@app_support_subfolder = paths[0].path.stringByExpandingTildeInPath + "/com.LumberjackSystem.Lumberyard"
			Dir.mkdir(@app_support_subfolder) unless File.directory?(@app_support_subfolder)
		end
		@new_event_fcpxml = @app_support_subfolder + "/Logged Event.fcpxml"
		@new_project_xmeml = @app_support_subfolder + "/Logged Clips.xml"
		enableDebugMenu(self)
		setTextMate(self)
	end

	def applicationWillFinishLaunching(sender)
		OSX::LetsMove.PFMoveToApplicationsFolderIfNecessary
	end

	def applicationDidFinishLaunching(sender)
		puts("applicationDidFinishLaunching")
		NSApp.setServicesProvider(self)
		@logged_in = false
		getCredentialsFromKeychain()
		get_uris()
		login(self)
		@BLDR_identifier = "com.LumberjackSystem.Builder"
		checkRunningAppsForNLEs()
		if @FCPX_identifier.nil? ### not running, but FCPX identifier is constant, so we can get the FCPX purchased or trial app
			@FCPX_identifier = 'com.apple.FinalCut'
			fcpx_app = NSWorkspace.sharedWorkspace.absolutePathForAppBundleWithIdentifier('com.apple.FinalCut')
			if fcpx_app.nil?
				fcpt_app = NSWorkspace.sharedWorkspace.absolutePathForAppBundleWithIdentifier('com.apple.FinalCutTrial')
				if fcpt_app!=nil
					@FCPX_identifier = 'com.apple.FinalCutTrial'
				end
			end
			#puts "@FCPX_identifier = #{@FCPX_identifier.inspect}" # will never be nil
		end
		# puts "openPanel"
		# @openPanel = OSX::NSOpenPanel.openPanel # retain as instance variables
		# @savePanel = OSX::NSSavePanel.savePanel
		@mainWindow.registerForDraggedTypes([OSX::NSPasteboardTypeString]) # @mainWindow delegate has been set in IB
	end

	def applicationDidBecomeActive(notification)
		checkRunningAppsForNLEs()
	end

	def checkRunningAppsForNLEs()
		runningApps = NSWorkspace.sharedWorkspace.runningApplications
		if @PPRO_identifier.nil?
			ppro = runningApps.find { |app| app.bundleIdentifier.to_s.include?("com.adobe.PremierePro") } # PPro identifier varies between versions, so we'll get the running identifier
			if ppro.nil?
				@PPRO_identifier = nil
			else
				@PPRO_identifier = ppro.bundleIdentifier.to_s
			end
		end
		if @FCPX_identifier.nil?
			fcpx = runningApps.find { |app| app.bundleIdentifier.to_s.include?("com.apple.FinalCut") } # could be trial version or not
			if fcpx.nil?
				@FCPX_identifier = nil
			else
				@FCPX_identifier = fcpx.bundleIdentifier.to_s
			end
		end
		#puts "@PPRO_identifier = #{@PPRO_identifier.inspect}", "@FCPX_identifier = #{@FCPX_identifier.inspect}", "@BLDR_identifier = #{@BLDR_identifier.inspect}"
	end
	
	def showPreferences(sender)
		getCredentialsFromKeychain() # in case they've been changed in Keychain Access
		@preferencesWindow.makeKeyAndOrderFront(self)
	end
	ib_action :showPreferences
	
	def showHelp(sender)
		nsURL = NSURL.URLWithString("http://lumberjacksystem.com/lumberyard.html")
		NSWorkspace.sharedWorkspace.openURL(nsURL)
	end
	ib_action :showHelp
	
	def emailSupport(sender)
		executable = OSX::NSBundle.mainBundle.infoDictionary.objectForKey("CFBundleExecutable")
		version = OSX::NSBundle.mainBundle.infoDictionary.objectForKey("CFBundleShortVersionString")
		app = "#{executable} (version #{version})".to_ns.stringByAddingPercentEscapesUsingEncoding(4)
		body = "What went wrong?\n\n\n".to_ns.stringByAddingPercentEscapesUsingEncoding(4)
		mailto = OSX::NSURL.URLWithString("mailto:info@LumberjackSystem.com?subject=Bug%20Report%20for%20#{app}&body=#{body}")
		result = OSX::NSWorkspace.sharedWorkspace.openURL(mailto)
		if result==false
			contactus = OSX::NSURL.URLWithString("http://lumberjacksystem.com/about.html")
			OSX::NSWorkspace.sharedWorkspace.openURL(contactus)
		end
	end
	ib_action :emailSupport
	
	def showGlossary(sender)
		nsURL = NSURL.URLWithString("http://lumberjacksystem.com/glossary.html")
		NSWorkspace.sharedWorkspace.openURL(nsURL)
	end
	ib_action :showGlossary
	
	def installMetadataView(sender) # copy Lumberjack metadata view.plist to ~/Library/Application Support/ProApps/Metadata Definitions/
		#puts "installMetadataView"
		bundle_plist_file = "#{NSBundle.mainBundle.bundlePath}/Contents/Resources/Lumberjack metadata view.plist"
		paths = NSFileManager.defaultManager.URLsForDirectory_inDomains(NSApplicationSupportDirectory, NSUserDomainMask)
		if paths.length > 0
			metadata_defs_folder = paths.first.path.stringByExpandingTildeInPath + "/ProApps/Metadata Definitions"
			madeFolder, error = NSFileManager.defaultManager.createDirectoryAtPath_withIntermediateDirectories_attributes_error(metadata_defs_folder, true, nil)
			metadata_defs_plist_file = "#{metadata_defs_folder}/Lumberjack metadata view.plist"
			NSFileManager.defaultManager.removeItemAtPath_error(metadata_defs_plist_file, nil)
			plist_installed, error = NSFileManager.defaultManager.copyItemAtPath_toPath_error(bundle_plist_file, metadata_defs_plist_file)
			#puts File.exist?(metadata_defs_plist_file)
			alert = NSAlert.alloc.init
			if plist_installed
				#puts "plist has been installed"
				alert.setMessageText("The Metadata View \"Lumberjack\" has been successfully installed")
				alert.setInformativeText("Please restart Final Cut Pro X to add this view to the Metadata View popup menu.")
			else
				alert.setMessageText("The Metadata View could not be installed")
			end
			alert.beginSheetModalForWindow_modalDelegate_didEndSelector_contextInfo(@mainWindow, self, nil, nil) # Deprecated after 10.10
		end
	end
	ib_action :installMetadataView
	
	def enableDebugMenu(sender)
		if @debugMenuCheckbox.state==1
			@debugMenu.setHidden(false)
		else
			@debugMenu.setHidden(true)
		end
	end
	ib_action :enableDebugMenu
	
	def showAppSupportFolder(sender)
		NSWorkspace.sharedWorkspace.openFile(@app_support_subfolder)
	end
	ib_action :showAppSupportFolder
	
	def setLocalhost(sender)
		if sender.class==NSMenuItem # state about to change...
			if @localhostMenuItem.state==1
				$localhost = false
			else
				$localhost = true
			end
		end
	end
	ib_action :setLocalhost

	def setTextMate(sender)
		if sender.class==NSMenuItem # state about to change...
			if @textmateMenuItem.state==1
				$textmate = false
			else
				$textmate = true
			end
		else # get state at startup
			if @debugMenuCheckbox.state==1 && @textmateMenuItem.state==1
				$textmate = true
			else
				$textmate = false
			end
		end
	end
	ib_action :setTextMate
	
	def getCredentialsFromKeychain
		# puts "getCredentialsFromKeychain"
		username = ""
		password = ""
		keychain = `security find-generic-password -s 'Lumberjack account details' -g 2>&1` # redirect STDERR to STDOUT
		parts = keychain.split("\n")
		acct_string = parts.find { |p| p.include?("acct") }
		pssw_string = parts.find { |p| p.include?("password:") }
		if acct_string.nil? || pssw_string.nil?
			@usernameField.setStringValue('')
			@passwordField.setStringValue('')
			# log the user out if logged in
			@logged_in = true
			login(self)
		else
			splitAcct = acct_string.split('"') # split on "
			if splitAcct.length > 3
				username = splitAcct.last
			end
			splitPw = pssw_string.split('"')
			if splitPw.length > 1
				password = splitPw.last
			end
			@usernameField.setStringValue(username)
			@passwordField.setStringValue(password)
		end
		if username.empty? || password.empty?
			@preferencesWindow.makeKeyAndOrderFront(self)
			@preferencesTabs.selectFirstTabViewItem(self)
		end
	end
	
	def saveCredentialsToKeychain(sender)
		username = @usernameField.stringValue.strip
		if username.include?("mailto:")
			username = username.sub("mailto:", "")
		end
		@usernameField.stringValue = username
		password = @passwordField.stringValue.strip
		@passwordField.stringValue = password
		`security delete-generic-password -s 'Lumberjack account details'`
		`security add-generic-password -a '#{username}' -s 'Lumberjack account details' -w '#{password}'`
		@preferencesWindow.orderOut(self)
	end
	ib_action :saveCredentialsToKeychain
		
	def login(sender)
		if @logged_in # log out
			begin
				if @logout.nil? # haven't gotten the URIs yet
					get_uris()
				end
				#puts @logout.inspect
				response = get_NSMutableURLRequest_response(@logout)
				#puts response.inspect
				lock = NSImage.imageNamed("NSLockLockedTemplate")
				@loginButton.setImage(lock)
				@logged_in = false
				@xmlName.setStringValue("")
				unlockInterface(false)
			rescue => e # no net connection, can't find server...
				#puts "...problem getting the response"
				message = e.message
				message = "Is your Mac connected to the internet?" if message=="NSInvalidArgumentException - data parameter is nil"
				@spinner.stopAnimation(self)
				alert = NSAlert.alloc.init
				alert.setMessageText("There was a problem signing in to the Lumberjack server")
				alert.setInformativeText(message)
				alert.setAlertStyle(NSCriticalAlertStyle)
				alert.beginSheetModalForWindow_modalDelegate_didEndSelector_contextInfo(@mainWindow, self, nil, nil) # Deprecated after 10.10
			end
		elsif @usernameField.stringValue.empty? || @passwordField.stringValue.empty?
			@preferencesWindow.makeKeyAndOrderFront(self)
			@preferencesTabs.selectFirstTabViewItem(self)
		else # log in
			get_uris()
			if $localhost!=false || $textmate
				alert = NSAlert.alloc.init
				if $localhost!=false
					alert.setMessageText("Debug is enabled:")
					alert.setInformativeText("server = #{$localhost}\ntextmate = #{$textmate}")
				else
					alert.setMessageText("Debug is enabled, but localhost server is not running: run MAMP and try again")
					alert.setInformativeText("server = lumberjacksystem.com\ntextmate = #{$textmate}")
				end
				alert.setAlertStyle(NSCriticalAlertStyle)
				alert.beginSheetModalForWindow_modalDelegate_didEndSelector_contextInfo(@mainWindow, self, nil, nil) # Deprecated after 10.10
			end
			@spinner.startAnimation(self)
			#puts "start to connect... $localhost? #{$localhost}... #{@login}"
			begin
				# uri = URI.parse(@login)
				# http = Net::HTTP.new(uri.host, uri.port)
				# if $localhost == false
				# 	http.use_ssl = true
				# 	http.verify_mode = OpenSSL::SSL::VERIFY_NONE
				# end
				# request = Net::HTTP::Post.new(uri.request_uri)
				# request.set_form_data({ "username" => @usernameField.stringValue, "password" => @passwordField.stringValue, "lumberyard" => 1 })
				# response = http.request(request)
				username = ERB::Util.url_encode(@usernameField.stringValue)
				password = ERB::Util.url_encode(@passwordField.stringValue)
				post = "username=#{username}&password=#{password}&lumberyard=1"
				#// https://tools.ietf.org/html/rfc3986#section-2.2
				# rfc3986Reserved = OSX::NSCharacterSet.characterSetWithCharactersInString("!*'();:@&=+$,/?#[]")
				# esc_str = str.to_ns.stringByAddingPercentEncodingWithAllowedCharacters(rfc3986Reserved.invertedSet)
				json_response = get_NSMutableURLRequest_response(@login, post)
				unless json_response.empty? # no username or password, so ignore
					message = json_response["message"]
					@user_id = json_response["user_id"]
					@customer_id = json_response["customer_id"]
					#puts "@user_id = #{@user_id}, @customer_id = #{@customer_id}"
					get_uris()
					# uri = URI.parse(@list_stories)
					# http = Net::HTTP.new(uri.host, uri.port)
					# if $localhost == false
					# 	http.use_ssl = true
					# 	http.verify_mode = OpenSSL::SSL::VERIFY_NONE
					# end
					# request = Net::HTTP::Get.new(uri.request_uri)
					# response = http.request(request)
					response = get_NSMutableURLRequest_response(@list_stories)
					puts "response = #{response.inspect}"
					if @user_id=="0" #|| response.body=="This is not a valid customer"
						raise message
					else #if response.message=="OK"
						lock = NSImage.imageNamed("NSLockUnlockedTemplate")
						@loginButton.setImage(lock)
						@logged_in = true
						unlockInterface(true)
						# get all the stories, create popup menu
						@stories = [ {"event" => ""} ] + response # hold this result just in case a story is added to the database later
						@eventPopup.removeAllItems
						@stories.each do |s| # @stories includes empty first entry
							# @eventPopup.addItemWithTitle(s["event"])
							@eventPopup.menu.addItemWithTitle_action_keyEquivalent(s["event"], nil, "") # add items to the menu instead of the popup button so that duplicate event names can be supported
						end
						@report = "--------------------------------------------\n"
						# @titlesCheckbox.setEnabled(false) # FIXME
						# @titlesCheckbox.setState(0)
						# make timezone popup menus (requires internet connection)
						# selected_country = NSUserDefaults.standardUserDefaults.stringForKey("TzCountry")
						# @tzCountryPopup.removeAllItems
						# @tzCountryPopup.addItemWithTitle("United States")
						# @countries = TZInfo::Country.all.sort
						# @countries.each { |c| @tzCountryPopup.addItemWithTitle(c.to_s) unless c.code=="US" }
						# unless selected_country.nil?
						# @tzCountryPopup.selectItemWithTitle(selected_country)
						# end
						# chooseCountry(self) # build the @tzCityPopup menu
						# else
						# 	raise response.message
					end
				end
			rescue => e # no net connection, can't find server...
				#puts "...problem getting the response"
				@spinner.stopAnimation(self)
				alert = NSAlert.alloc.init
				alert.setMessageText("There was a problem signing in to the Lumberjack server")
				#puts e.message
				alert.setInformativeText(e.message)
				alert.setAlertStyle(NSCriticalAlertStyle)
				alert.beginSheetModalForWindow_modalDelegate_didEndSelector_contextInfo(@mainWindow, self, nil, nil) # Deprecated after 10.10
			end
			@spinner.stopAnimation(self)
		end
	end
	ib_action :login
	
	def unlockInterface(bool)
		@importXMLButton.setEnabled(bool)
		@eventPopup.setEnabled(bool)
		#cardPopup
		@posNegPopup.setEnabled(bool)
		@dayPopup.setEnabled(bool)
		@hrsPopup.setEnabled(bool)
		@minPopup.setEnabled(bool)
		@secPopup.setEnabled(bool)
		@keywordsTableView.setEnabled(bool)
		@mediaTableView.setEnabled(bool)
	end

	def open_fcpxml(sender)
		oPanel = NSOpenPanel.openPanel
		oPanel.setMessage("Choose an XML of Clips exported from Final Cut Pro or Premiere Pro")
		oPanel.setAllowsMultipleSelection(false)
		types = ["fcpxml", "fcpxmld", "xml"]
		oPanel.beginSheetForDirectory_file_types_modalForWindow_modalDelegate_didEndSelector_contextInfo(nil, nil, types, @mainWindow, self, 'didEndOpenSheet:returnCode:contextInfo:', nil)
		# oPanel.beginSheetModalForWindow_completionHandler(@mainWindow, nil, nil)
		# result = oPanel.runModal
		# if result==NSFileHandlingPanelOKButton
		# 	oPanel.orderOut(self)
		# 	application_openFile(self, oPanel.URL.path.to_s)
		# end
	end
	ib_action :open_fcpxml
	
	def didEndOpenSheet_returnCode_contextInfo(oPanel, buttonClicked, info)
		if buttonClicked==1
			oPanel.orderOut(self)
			if oPanel.URL.pathExtension=="xml"
				application_openFile(self, oPanel.URL.path, "xmeml")
			else
				application_openFile(self, oPanel.URL.path, "path")
			end
		end
	end
	
	def draggingEntered(sender) # drag to app window
		pasteBoard = sender.draggingPasteboard
		if pasteBoard.types.include?("com.apple.finalcutpro.xml")
			OSX::NSCursor.dragCopyCursor.set
			return OSX::NSDragOperationCopy
		else
			OSX::NSCursor.arrowCursor.set
			return OSX::NSDragOperationNone
		end
	end
	
	def draggingExited(sender)
		OSX::NSCursor.arrowCursor.set
	end
	
	# def prepareForDragOperation(sender)
	#   true
	# end
	
	def performDragOperation(sender) # drag-and-drop to app window
		OSX::NSCursor.arrowCursor.set
		pasteBoard = sender.draggingPasteboard
		if pasteBoard.types.include?("com.apple.finalcutpro.xml")
			fcpxml = pasteBoard.stringForType("com.apple.finalcutpro.xml")
			application_openFile(self, fcpxml, "xml")
			return true
		end
		return false
	end
	
	def draggedEventItems(pBoard) # drag-and-drop to dock icon
		pasteBoard = NSPasteboard.pasteboardWithName(NSDragPboard)
		if pasteBoard.types.include?("com.apple.finalcutpro.xml")
			fcpxml = pasteBoard.stringForType("com.apple.finalcutpro.xml")
			application_openFile(self, fcpxml, "xml")
		else
			@xmlIcon.setHidden(true)
			@xmlIcon.setToolTip("")
			@xmlFileName.setStringValue("")
			@clipCount.setStringValue("")
			@saveFileButton.setEnabled(false)
			@saveMenuItem.setEnabled(false)
			@openFileButton.setEnabled(true)
		end
	end

	def application_openFile(sender, obj, type="path")
		#puts "application_openFile"
		if type=="xmeml"
			@XML_flavor = "xmeml"
			parse_xmeml(obj)
		else
			@XML_flavor = "fcpxml"
			parse_fcpxml(obj, type)
		end
		return true
	end
	
	def parse_fcpxml(obj, type)
		@NLE_identifier = @FCPX_identifier
		@sendToNLEButton.setTitle("Send to Final Cut Pro")
		@spinner.startAnimation(self)
		@progressIndicator.setIndeterminate(true)
		@progressIndicator.setUsesThreadedAnimation(true)
		@progressIndicator.startAnimation(self)
		@progressMessage.setStringValue("Working...\n")
		@progressView.displayIfNeeded
		NSApp.beginSheet_modalForWindow_modalDelegate_didEndSelector_contextInfo(@progressPanel, @mainWindow, self, nil, nil)
		if type=="path" # file path from opened or dragged .fcpxml
			NSDocumentController.sharedDocumentController.noteNewRecentDocumentURL( NSURL.fileURLWithPath_isDirectory(obj, false) ) # add this file to the Recent menu
			path = obj.to_s
			@xmlName.setStringValue(File.basename(path))
			# parse event XML
			if path.hasSuffix?(".fcpxml")
				@nokogiri_xml = Nokogiri.XML(File.open(obj)) { |config| config.huge.noblanks }
			else
				@nokogiri_xml = Nokogiri.XML(File.open("#{obj}/Info.fcpxml")) { |config| config.huge.noblanks }
			end # FIXME deal with invalid characters
		else # pasteboard dragged event items fcpxml
			xml = obj.to_s
			@nokogiri_xml = Nokogiri.XML(xml) { |config| config.huge.noblanks }
			if @nokogiri_xml.at_xpath("fcpxml/library")!=nil
				library_name = "Dragged Library from Final Cut Pro X"
				location = @nokogiri_xml.at_xpath("fcpxml/library")["location"]
				url = NSURL.URLWithString(location)
				unless url.nil?
					library_name = url.pathComponents.last.sub(".fcpbundle", "")
				end
				@xmlName.setStringValue(library_name)
			elsif @nokogiri_xml.at_xpath("fcpxml/event")!=nil
				event_name = @nokogiri_xml.at_xpath("fcpxml/event")["name"]
				if event_name.empty?
					@xmlName.setStringValue("Dragged Event from Final Cut Pro X")
				else
					@xmlName.setStringValue(event_name)
				end
			else # naked items not in an Event
				@xmlName.setStringValue("")
				@spinner.stopAnimation(self)
				@assets = []
				@mediaTableView.reloadData
				alert = OSX::NSAlert.alloc.init
				alert.setAlertStyle(OSX::NSInformationalAlertStyle)
				alert.setMessageText("Drag an Event or Library to the Lumberyard window")
				alert.setInformativeText("Lumberyard cannot work with clips or multicams dragged to its window")
				alert.beginSheetModalForWindow_modalDelegate_didEndSelector_contextInfo(@mainWindow, self, nil, nil) # Deprecated after 10.10
			end
		end
		@xml_string = @nokogiri_xml.to_s
		@report = "--------------------------------------------\n"
		make_assets_database_from_resources()
		@clips = Array.new
		#puts "### CONVERT KEYWORDS TO METADATA LOGS ###"
		locations, people, activities, others = [], [], [], []
		@nokogiri_xml.xpath(".//collection-folder [@name='Locations—Lumberjack']/keyword-collection").each { |kc| locations << kc["name"] }
		@nokogiri_xml.xpath(".//collection-folder [@name='People—Lumberjack']/keyword-collection").each { |kc| people << kc["name"] }
		@nokogiri_xml.xpath(".//collection-folder [@name='Activities—Lumberjack']/keyword-collection").each { |kc| activities << kc["name"] }
		@nokogiri_xml.xpath(".//collection-folder [@name='Others—Lumberjack']/keyword-collection").each { |kc| others << kc["name"] }
		unsorted_metadata_logs = Array.new
		@event_clips_nodeset = @nokogiri_xml.xpath(Event_MCclip_XPath) + @nokogiri_xml.xpath(Event_Clip_XPath) # process mc-clip before others so we can determine if an asset is part of a multicam (and its clip can be ignored for string-outs)
		#puts "#{@event_clips_nodeset.length} @event_clips_nodeset"
		@progressIndicator.setIndeterminate(false)
		@progressIndicator.setDoubleValue(0)
		@progressIndicator.setMaxValue(@event_clips_nodeset.length)
		@progressMessage.setStringValue("Analyzing clips and multicams...\n")
		@progressView.displayIfNeeded
		@event_clips_nodeset.each do |clip_node|
			@progressIndicator.incrementBy(1)
			@progressIndicator.displayIfNeeded
			# first add frame and timecode information to assets
			unless clip_node.parent.name=="mc-angle"
				day = @dayPopup.titleOfSelectedItem.to_i
				hrs = @hrsPopup.titleOfSelectedItem.to_i
				min = @minPopup.titleOfSelectedItem.to_i
				sec = @secPopup.titleOfSelectedItem.to_i
				if @posNegPopup.titleOfSelectedItem=="+"
					adj_time = day*24*60*60 + hrs*60*60 + min*60 + sec # seconds
				else
					adj_time = 0 - (day*24*60*60 + hrs*60*60 + min*60 + sec)
				end
				if clip_node.name=="clip" || clip_node.name=="asset-clip" || clip_node.name=="sync-clip"
					format = @nokogiri_xml.at_xpath(".//resources/format[@id='#{clip_node["format"]}']")
					if format.nil? # still images have no format
						refs = []
					else
						format["frameDuration"] = "1/60s" if format["frameDuration"].nil?
						clip_node["tcFormat"] = "NDF" if clip_node["tcFormat"].nil? # audio-only clips have no tcFormat which is a problem for string-out sequences
						tcFormat = clip_node["tcFormat"]
						case clip_node.name
						when "clip"
							refs = clip_node.xpath(".//video")
						when "asset-clip" # ref in asset-clip
							refs = [ clip_node ]
						when "sync-clip"
							refs = clip_node.xpath(".//clip/video | .//asset-clip")
						else
							refs = []
						end
					end
				elsif clip_node.name=="mc-clip"
					multicam_node = @nokogiri_xml.at_xpath(".//resources/media[@id='#{clip_node["ref"]}']/multicam")
					if multicam_node.nil? # media element with no children
						puts "multicam_node is nil!!!"
						refs = []
					else
						format = @nokogiri_xml.at_xpath(".//resources/format[@id='#{multicam_node["format"]}']")
						tcFormat = multicam_node["tcFormat"]
						refs = @nokogiri_xml.xpath( ".//resources/media[@id='#{clip_node["ref"]}']/multicam/mc-angle/clip/video | .//resources/media[@id='#{clip_node["ref"]}']/multicam/mc-angle/asset-clip")
					end
				end
				puts "Working on asset for #{clip_node["name"]}..."
				if refs.empty? # audio-only clip or still image
					audios = clip_node.xpath("audio")
					if audios.empty?
						ref = nil
					else
						ref = audios.last
					end
				else
					ref = refs.first
				end
				if ref.nil?
					asset = nil
				else
					asset = @assets.find { |a| a[:id]==ref["ref"] }
				end
				unless asset.nil?
					if clip_node.name=="asset-clip" # dtd 1.6
						clip_node["start"] = asset["start"]
						clip_node["duration"] = asset["duration"]
					end
					clip_hash = asset.dup
					clip_hash["clip_name"] = clip_node["name"]
					#pp clip_hash
					clip_hash[:tcFormat] = tcFormat
					frameDuration = to_rational(format["frameDuration"])
					clip_hash[:timebase] = 1 / frameDuration
					clip_hash[:first_frame] = to_frames(clip_node["start"], format["frameDuration"], tcFormat)
					clip_hash[:duration_frames] = to_frames(clip_node["duration"], format["frameDuration"], tcFormat)
					clip_hash[:start_frame] = clip_hash[:first_frame] + to_frames(adj_time.to_s, format["frameDuration"], tcFormat)
					clip_hash[:stop_frame] = clip_hash[:first_frame] + clip_hash[:duration_frames] + to_frames(adj_time.to_s, format["frameDuration"], tcFormat)
					clip_hash["start_timecode"] = to_timecode(clip_hash[:start_frame], clip_hash[:timebase], clip_hash[:tcFormat])
					clip_hash["stop_timecode"] = to_timecode(clip_hash[:stop_frame], clip_hash[:timebase], clip_hash[:tcFormat])
					@clips << clip_hash
				end
			end
			# then get clip's existing keywords and ratings
			clip_node.xpath("keyword | rating").each do |keyword|
				keyword["value"].split(", ").each do |value|
					lumberjack_keyword = false
					if locations.include?(value)
						keyword_name = "l:#{value}"
						lumberjack_keyword = true
					elsif people.include?(value)
						keyword_name = "p:#{value}"
						lumberjack_keyword = true
					elsif activities.include?(value)
						keyword_name = "a:#{value}"
						lumberjack_keyword = true
					elsif others.include?(value)
						keyword_name = "k:#{value}"
						lumberjack_keyword = true
					elsif value=="favorite"
						keyword_name = "favorite"
						lumberjack_keyword = true
					end
					if lumberjack_keyword
						start_time = 0 + to_rational(keyword["start"])
						stop_time = start_time + to_rational(keyword["duration"])
						if clip_node.name=="clip" || clip_node.name=="asset-clip" || clip_node.name=="sync-clip"
							format = @nokogiri_xml.at_xpath(".//resources/format[@id='#{clip_node["format"]}']")
							if format.nil? # still images have no format
								refs = []
							else
								format["frameDuration"] = "1/60s" if format["frameDuration"].nil?
								clip_node["tcFormat"] = "NDF" if clip_node["tcFormat"].nil? # audio-only clips have no tcFormat which is a problem for string-out sequences
								tcFormat = clip_node["tcFormat"]
								case clip_node.name
								when "clip"
									refs = clip_node.xpath(".//video")
								when "asset-clip" # ref in asset-clip
									refs = [ clip_node ]
								when "sync-clip"
									refs = clip_node.xpath(".//clip/video | .//asset-clip")
								else
									refs = []
								end
							end
						elsif clip_node.name=="mc-clip"
							multicam_node = @nokogiri_xml.at_xpath(".//resources/media[@id='#{clip_node["ref"]}']/multicam")
							format = @nokogiri_xml.at_xpath(".//resources/format[@id='#{multicam_node["format"]}']")
							tcFormat = multicam_node["tcFormat"]
							refs = @nokogiri_xml.xpath( ".//resources/media[@id='#{clip_node["ref"]}']/multicam/mc-angle/clip/video | .//resources/media[@id='#{clip_node["ref"]}']/multicam/mc-angle/asset-clip")
						end
						#puts "Working on asset for #{clip_node["name"]}..."
						if refs.empty? # audio-only clip or still image
							audios = clip_node.xpath("audio")
							if audios.empty?
								ref = nil
							else
								ref = audios.last
							end
						else
							ref = refs.first
						end
						if ref.nil?
							asset = nil
						else
							asset = @assets.find { |a| a[:id]==ref["ref"] }
						end
						if asset.nil?
							local_start_time = local_stop_time = ""
						else
							clip_start = to_rational(clip_node["start"])
							utc_time = asset[:start_time] + (start_time - clip_start)
							#puts "utc_time = #{asset[:start_time]} + (#{start_time} - #{clip_start}) = #{utc_time}"
							localizedDate = NSDateFormatter.localizedStringFromDate_dateStyle_timeStyle(utc_time.to_ns, NSDateFormatterShortStyle, NSDateFormatterMediumStyle)
							local_start_time = localizedDate
							utc_time = asset[:start_time] + (stop_time - clip_start)
							#puts "utc_time = #{asset[:start_time]} + (#{stop_time} - #{clip_start}) = #{utc_time}"
							localizedDate = NSDateFormatter.localizedStringFromDate_dateStyle_timeStyle(utc_time.to_ns, NSDateFormatterShortStyle, NSDateFormatterMediumStyle)
							local_stop_time = localizedDate
						end
						unsorted_metadata_logs << { "metadata" => keyword_name, "start_time" => start_time, "local_start_time" => local_start_time, "stop_time" => stop_time, "local_stop_time" => local_stop_time }
					end
				end
			end
		end
		@metadata_logs = unsorted_metadata_logs.sort_by { |m| m["start_time"] }
		# determine if event is already logged with keywords
		keyword_logs = @metadata_logs.find_all { |m| m["metadata"]!="favorite" }
		if keyword_logs.empty? # an event with only favorites does not trigger Story Mode
			@events_already_logged = false
		else
			@events_already_logged = true
		end
		#puts "@assets: #{@assets.length} items"
		if @assets.empty? && @missing_media #&& @metadata_logs.empty?
			alert = NSAlert.alloc.init
			alert.setMessageText("There was a problem with this Event XML file")
			alert.setInformativeText("Missing media files - media in this Event is offline. \nTry exporting the Event XML again.")
			alert.setAlertStyle(NSCriticalAlertStyle)
			alert.beginSheetModalForWindow_modalDelegate_didEndSelector_contextInfo(@mainWindow, self, nil, nil) # Deprecated after 10.10
			@transcriptsButton.setEnabled(false)
			@sendToNLEButton.setEnabled(false)
			@sendToBuilderButton.setEnabled(false)
		elsif @events_already_logged
			#puts "story mode: #{@metadata_logs.count} log rows"
			@mode = "Story"
			@mainWindow.setTitle("Lumberyard — Story Mode")
			@eventPopup.removeAllItems
			@eventPopup.setEnabled(false)
			@transcriptsButton.setEnabled(true)
			@posNegPopup.setEnabled(false)
			@posNegPopup.selectItemAtIndex(0)
			@dayPopup.setEnabled(false)
			@dayPopup.selectItemAtIndex(0)
			@hrsPopup.setEnabled(false)
			@hrsPopup.selectItemAtIndex(0)
			@minPopup.setEnabled(false)
			@minPopup.selectItemAtIndex(0)
			@secPopup.setEnabled(false)
			@secPopup.selectItemAtIndex(0)
			chooseEvent(self) # build @people, @activities, @keywords, @favorites arrays
		else # @metadata_logs.empty? or not...
			@mode = ""
			if @eventPopup.numberOfItems==0
				@eventPopup.removeAllItems
				@stories.each do |s| # @stories includes empty first entry
					# @eventPopup.addItemWithTitle(s["event"])
					@eventPopup.menu.addItemWithTitle_action_keyEquivalent(s["event"], nil, "") # add items to the menu instead of the popup button so that duplicate event names can be supported
				end
			end
			@eventPopup.setEnabled(true)
			@transcriptsButton.setEnabled(true)
			@posNegPopup.setEnabled(true)
			@dayPopup.setEnabled(true)
			@hrsPopup.setEnabled(true)
			@minPopup.setEnabled(true)
			@secPopup.setEnabled(true)
			@keywordsTableView.setEnabled(true)
			@mediaTableView.setEnabled(true)
			chooseEvent(self)
		end
		@mediaTableView.reloadData
		@keywordsTableView.reloadData
		@spinner.stopAnimation(self)
		@progressMessage.setStringValue("")
		@progressIndicator.stopAnimation(self)
		@mainWindow.endSheet(@progressPanel) # NSApp.endSheet(@progressPanel)
		@progressPanel.orderOut(nil)
	end

	def parse_xmeml(obj)
		@NLE_identifier = @PPRO_identifier
		@sendToNLEButton.setTitle("Send to Premiere Pro")
		@spinner.startAnimation(self)
		@progressIndicator.setIndeterminate(true)
		@progressIndicator.setUsesThreadedAnimation(true)
		@progressIndicator.startAnimation(self)
		@progressMessage.setStringValue("Working...\n")
		@progressView.displayIfNeeded
		NSApp.beginSheet_modalForWindow_modalDelegate_didEndSelector_contextInfo(@progressPanel, @mainWindow, self, nil, nil)
		NSDocumentController.sharedDocumentController.noteNewRecentDocumentURL( NSURL.fileURLWithPath_isDirectory(obj, false) ) # add this file to the Recent menu
		path = obj.to_s
		@xmlName.setStringValue(File.basename(path))
		# parse xmeml
		f = File.open(path)
		@nokogiri_xml = Nokogiri.XML(f) { |config| config.recover.huge }
		f.close
		@xml_string = @nokogiri_xml.to_s
		@report = "--------------------------------------------\n"
		make_assets_database_from_files()
		# translate_multicamseqs_to_multiclips() # we used to translate the sequence to an old multiclip structure, but now we just leave them as sequences which can be turned back into multiclips in PPro
		@clips = Array.new
		unsorted_metadata_logs = Array.new
		# @bin_clips_nodeset = @nokogiri_xml.xpath(Bin_Clip_XPath) # will include new multiclips
		@bin_clips_nodeset = @nokogiri_xml.xpath(Bin_Clip_And_MCSeq_XPath)
		@progressIndicator.setIndeterminate(false)
		@progressIndicator.setDoubleValue(0)
		@progressIndicator.setMaxValue(@bin_clips_nodeset.length)
		@progressMessage.setStringValue("Analyzing clips and multicams...\n")
		@progressView.displayIfNeeded
		@bin_clips_nodeset.each do |clip_node|
			@progressIndicator.incrementBy(1)
			@progressIndicator.displayIfNeeded
			timebase = 1
			asset = nil
			# first add frame and timecode information to assets
			unless clip_node.parent.name=="mc-angle"
				day = @dayPopup.titleOfSelectedItem.to_i
				hrs = @hrsPopup.titleOfSelectedItem.to_i
				min = @minPopup.titleOfSelectedItem.to_i
				sec = @secPopup.titleOfSelectedItem.to_i
				if @posNegPopup.titleOfSelectedItem=="+"
					adj_time = day*24*60*60 + hrs*60*60 + min*60 + sec # seconds
				else
					adj_time = 0 - (day*24*60*60 + hrs*60*60 + min*60 + sec)
				end
				# if clip_node.name=="clip"
					file_id = clip_node.at_xpath(".//file")["id"]
					tcFormat = @nokogiri_xml.at_xpath(".//file[@id='#{file_id}']/timecode/displayformat").inner_text
					timebase = clip_node.at_xpath("rate/timebase").inner_text.to_i
					ntsc = clip_node.at_xpath("rate/ntsc").inner_text
				# elsif clip_node.name=="mc-clip"
				# end
				#puts "Working on asset for #{clip_node["name"]}..."
				asset = @assets.find { |a| a[:id]==file_id }
				unless asset.nil?
					clip_hash = asset.dup
					clip_name = clip_node.at_xpath("name").inner_text
					clip_hash["clip_name"] = clip_name
					# pp clip_hash
					clip_hash[:tcFormat] = tcFormat
					if ntsc=="TRUE"
						timebase_rational = Rational.new!(timebase * 1000, 1001)
					else
						timebase_rational = Rational.new!(timebase * 1000, 1000)
					end
					frame_duration = 1 / timebase_rational
					clip_hash[:timebase] = timebase_rational
					clip_hash[:first_frame] = asset["start"]
					clip_hash[:duration_frames] = asset["duration"]
					clip_hash[:start_frame] = clip_hash[:first_frame] + to_frames(adj_time.to_s, frame_duration.to_s, tcFormat)
					clip_hash[:stop_frame] = clip_hash[:first_frame] + clip_hash[:duration_frames] + to_frames(adj_time.to_s, frame_duration.to_s, tcFormat)
					clip_hash["start_timecode"] = to_timecode(clip_hash[:start_frame], clip_hash[:timebase], clip_hash[:tcFormat])
					clip_hash["stop_timecode"] = to_timecode(clip_hash[:stop_frame], clip_hash[:timebase], clip_hash[:tcFormat])
					@clips << clip_hash
				end
			end
			# then get clip's existing markers
			clip_node.xpath("marker").each do |marker|
				name = marker.at_xpath("name").inner_text
				color = marker.at_xpath("pproColor").inner_text unless marker.at_xpath("pproColor").nil?
				in_point = marker.at_xpath("in").inner_text.to_i
				out_point = marker.at_xpath("out").inner_text.to_i
				duration = (out_point - in_point) / timebase
				if out_point > 0 && duration > 5
					keyword_name = ""
					case color
					when Blue_Location
						keyword_name = "l:#{name}"
					when Purple_Person
						keyword_name = "p:#{name}"
					when Yellow_Activity
						keyword_name = "a:#{name}"
					when Orange_Other
						keyword_name = "k:#{name}"
					end
					unless keyword_name.empty?
						start_time = in_point / timebase
						stop_time = out_point / timebase
						if asset.nil?
							local_start_time = local_stop_time = ""
						else
							utc_time = asset[:start_time] + start_time
							#puts "utc_time = #{asset[:start_time]} + #{start_time} = #{utc_time}"
							localizedDate = NSDateFormatter.localizedStringFromDate_dateStyle_timeStyle(utc_time.to_ns, NSDateFormatterShortStyle, NSDateFormatterMediumStyle)
							local_start_time = localizedDate
							utc_time = asset[:start_time] + stop_time
							#puts "utc_time = #{asset[:start_time]} + #{stop_time} = #{utc_time}"
							localizedDate = NSDateFormatter.localizedStringFromDate_dateStyle_timeStyle(utc_time.to_ns, NSDateFormatterShortStyle, NSDateFormatterMediumStyle)
							local_stop_time = localizedDate
						end
						unsorted_metadata_logs << { "metadata" => keyword_name, "start_time" => start_time, "local_start_time" => local_start_time, "stop_time" => stop_time, "local_stop_time" => local_stop_time }
					end
				end
			end
		end
		#puts "@clips:"
		#pp @clips
		@metadata_logs = unsorted_metadata_logs.sort_by { |m| m["start_time"] }
		# determine if event is already logged with keywords
		keyword_logs = @metadata_logs.find_all { |m| m["metadata"]!="favorite" }
		if keyword_logs.empty? # an event with only favorites does not trigger Story Mode
			@events_already_logged = false
		else
			@events_already_logged = true
		end
		#puts "@events_already_logged = #{@events_already_logged}"
		#puts "@assets: #{@assets.length} items"
		if @assets.empty? #&& @metadata_logs.empty?
			alert = NSAlert.alloc.init
			alert.setMessageText("There was a problem with this Event XML file")
			alert.setInformativeText("Missing media files - media in this Event is offline. \nTry exporting the Event XML again.")
			alert.setAlertStyle(NSCriticalAlertStyle)
			alert.beginSheetModalForWindow_modalDelegate_didEndSelector_contextInfo(@mainWindow, self, nil, nil) # Deprecated after 10.10
			@transcriptsButton.setEnabled(false)
			@sendToNLEButton.setEnabled(false)
			@sendToBuilderButton.setEnabled(false)
		elsif @events_already_logged
			#puts "story mode: #{@metadata_logs.count} log rows"
			@mode = "Story"
			@mainWindow.setTitle("Lumberyard — Story Mode")
			@eventPopup.removeAllItems
			@eventPopup.setEnabled(false)
			@transcriptsButton.setEnabled(true)
			@posNegPopup.setEnabled(false)
			@posNegPopup.selectItemAtIndex(0)
			@dayPopup.setEnabled(false)
			@dayPopup.selectItemAtIndex(0)
			@hrsPopup.setEnabled(false)
			@hrsPopup.selectItemAtIndex(0)
			@minPopup.setEnabled(false)
			@minPopup.selectItemAtIndex(0)
			@secPopup.setEnabled(false)
			@secPopup.selectItemAtIndex(0)
			chooseEvent(self) # build @people, @activities, @keywords, @favorites arrays
		else # @metadata_logs.empty? or not...
			@mode = ""
			if @eventPopup.numberOfItems==0
				@eventPopup.removeAllItems
				@stories.each do |s| # @stories includes empty first entry
					# @eventPopup.addItemWithTitle(s["event"])
					@eventPopup.menu.addItemWithTitle_action_keyEquivalent(s["event"], nil, "") # add items to the menu instead of the popup button so that duplicate event names can be supported
				end
			end
			@eventPopup.setEnabled(true)
			@transcriptsButton.setEnabled(true)
			@posNegPopup.setEnabled(true)
			@dayPopup.setEnabled(true)
			@hrsPopup.setEnabled(true)
			@minPopup.setEnabled(true)
			@secPopup.setEnabled(true)
			@keywordsTableView.setEnabled(true)
			@mediaTableView.setEnabled(true)
			chooseEvent(self)
		end
		@mediaTableView.reloadData
		@keywordsTableView.reloadData
		@spinner.stopAnimation(self)
		@progressMessage.setStringValue("")
		@progressIndicator.stopAnimation(self)
		@mainWindow.endSheet(@progressPanel) # NSApp.endSheet(@progressPanel)
		@progressPanel.orderOut(nil)
	end

	def chooseEvent(sender)
		#puts "chooseEvent"
		@mediaTableView.tableColumns[0].headerCell.setStringValue("Clips")
		@mediaTableView.tableColumns[1].headerCell.setStringValue("Start Time")
		@mediaTableView.tableColumns[2].headerCell.setStringValue("Stop Time")
		@mediaTableView.reloadData
		@keywordsTableView.tableColumns[0].headerCell.setStringValue("Logged Keywords")
		@keywordsTableView.tableColumns[1].headerCell.setStringValue("Start Time")
		@keywordsTableView.tableColumns[2].headerCell.setStringValue("Stop Time")
		@keywordsTableView.reloadData
		@peopleActivitiesCheckbox.setTitle("each person's activities")
		@peopleKeywordsCheckbox.setTitle("each person's other keywords")
		@peopleTranscriptsCheckbox.setTitle("each person's transcripts")
		@activitesCheckbox.setTitle("each activity")
		@keywordsCheckbox.setTitle("each other keyword")
		@favoritesCheckbox.setTitle("favorite ranges")
		load_metadata_logs_for_event()
	end
	ib_action :chooseEvent
	
	def load_metadata_logs_for_event
		# puts "load_metadata_logs_for_event"
		story = @stories[@eventPopup.indexOfSelectedItem]
		#puts @eventPopup.indexOfSelectedItem
		#pp story
		@spinner.startAnimation(self)
		# Thread.new do
		unless @events_already_logged
			if @eventPopup.indexOfSelectedItem==0
				@metadata_logs = []
				@mode = ""
			else
				@mode = "Keywords"
				@mainWindow.setTitle("Lumberyard — Keywords Mode")
				# uri = URI.parse("#{@list_metadata_logs}/#{story["id"]}")
				# http = Net::HTTP.new(uri.host, uri.port)
				# if $localhost == false
				# 	http.use_ssl = true
				# 	http.verify_mode = OpenSSL::SSL::VERIFY_NONE
				# end
				# request = Net::HTTP::Get.new(uri.request_uri)
				# response = http.request(request)
				# if response.message=="OK"
				# 	unsorted_metadata_logs = JSON.load(response.body)
				# 	@metadata_logs = unsorted_metadata_logs.sort_by { |m| m["start_time"] }
				# else
				# 	raise response.message
				# end
				unsorted_metadata_logs = get_NSMutableURLRequest_response("#{@list_metadata_logs}/#{story["id"]}")
				@metadata_logs = unsorted_metadata_logs.sort_by { |m| m["start_time"] }
			end
		end
		# end # Thread
		#puts "#{@metadata_logs.count} log rows"
		@people, @activities, @keywords, @favorites = Array.new, Array.new, Array.new, Array.new
		server_timezone = TZInfo::Timezone.get("America/Los_Angeles")
		@metadata_logs.each do |m|
			if m["text"].class==NSNull
				m["text"] = ""
			end
			case m["metadata"][0,2]
			when "p:"
				@people << m["metadata"][2..-1] # whole string except first 2 characters
			when "a:"
				@activities << m["metadata"][2..-1]
			when "k:"
				@keywords << m["metadata"][2..-1]
			when "fa"
				@favorites << 1
			when "n:"
				m["keyword"] = m["metadata"][2..-1]
				m["metadata"] = "n:#{m["keyword"]} | #{m["text"]}"
			end
			unless @events_already_logged
				# convert Los Angeles timestamp strings (with no time zone) to UTC Date/Time - server time is set to be LA instead of just using Time.parse which uses computer's local time zone
				utc_time = server_timezone.local_to_utc(Time.parse(m["start_time"]))
				localizedDate = NSDateFormatter.localizedStringFromDate_dateStyle_timeStyle(utc_time.to_ns, NSDateFormatterShortStyle, NSDateFormatterMediumStyle)
				#puts "LA time: #{m["start_time"]}, UTC time: #{utc_time}, Local time: #{localizedDate}"
				m["local_start_time"] = localizedDate
				if m["stop_time"].nil? || m["stop_time"]=="0000-00-00 00:00:00" || m["stop_time"].class==NSNull # forgot to stop the logging
					m["local_stop_time"] = ""
					@report << "Log #{m["metadata"]} starting at #{localizedDate} was ignored because it has no stop time\n"
					@report << "--------------------------------------------\n"
				else
					utc_time = server_timezone.local_to_utc(Time.parse(m["stop_time"]))
					localizedDate = NSDateFormatter.localizedStringFromDate_dateStyle_timeStyle(utc_time.to_ns, NSDateFormatterShortStyle, NSDateFormatterMediumStyle)
					m["local_stop_time"] = localizedDate
				end
			end
		end
		#puts "------------------------------ @metadata_logs:"
		#pp @metadata_logs
		@keywordsTableView.reloadData
		@people.uniq!
		@people.sort!
		@activities.uniq!
		@activities.sort!
		@keywords.uniq!
		@keywords.sort!
		stringOutsOrNot(self)
		@spinner.stopAnimation(self)
	end
	
	def stringOutsOrNot(sender)
		#puts "stringOutsOrNot"
		if @people.length==1
			@peopleActivitiesCheckbox.setTitle("each person's activities (1 person)")
			@peopleKeywordsCheckbox.setTitle("each person's other keywords (1 person)")
			@peopleTranscriptsCheckbox.setTitle("each person's transcripts (1 person)")
		else
			@peopleActivitiesCheckbox.setTitle("each person's activities (#{@people.length} people)")
			@peopleKeywordsCheckbox.setTitle("each person's other keywords (#{@people.length} people)")
			@peopleTranscriptsCheckbox.setTitle("each person's transcripts (#{@people.length} people)")
		end
		if @people.empty?
			@peopleActivitiesCheckbox.setState(0)
			@peopleActivitiesCheckbox.setEnabled(false)
			@peopleKeywordsCheckbox.setState(0)
			@peopleKeywordsCheckbox.setEnabled(false)
			@peopleTranscriptsCheckbox.setState(0)
			@peopleTranscriptsCheckbox.setEnabled(false)
		elsif @mode=="Transcript"
			@peopleActivitiesCheckbox.setState(0)
			@peopleActivitiesCheckbox.setEnabled(false)
			@peopleKeywordsCheckbox.setState(0)
			@peopleKeywordsCheckbox.setEnabled(false)
			@peopleTranscriptsCheckbox.setEnabled(true)
		else
			@peopleActivitiesCheckbox.setEnabled(true)
			@peopleKeywordsCheckbox.setEnabled(true)
			@peopleTranscriptsCheckbox.setState(0)
			@peopleTranscriptsCheckbox.setEnabled(false)
		end
		if @activities.length==1
			@activitesCheckbox.setTitle("each activity (1 activity)")
		else
			@activitesCheckbox.setTitle("each activity (#{@activities.length} activities)")
		end
		if @activities.empty?
			@activitesCheckbox.setState(0)
			@activitesCheckbox.setEnabled(false)
		else
			@activitesCheckbox.setEnabled(true)
		end
		if @mode=="Transcript"
			if @keywords.length==1 # repurpose "other"
				@keywordsCheckbox.setTitle("each magic keyword (1 keyword)")
			else
				@keywordsCheckbox.setTitle("each magic keyword (#{@keywords.length} keywords)")
			end
		else # not Transcript
			if @keywords.length==1
				@keywordsCheckbox.setTitle("each other keyword (1 keyword)")
			else
				@keywordsCheckbox.setTitle("each other keyword (#{@keywords.length} keywords)")
			end
		end
		if @keywords.empty?
			@keywordsCheckbox.setState(0)
			@keywordsCheckbox.setEnabled(false)
		else
			@keywordsCheckbox.setEnabled(true)
		end
		@favoritesCheckbox.setTitle("#{@favorites.length} favorite ranges")
		if @favorites.empty?
			@favoritesCheckbox.setState(0)
			@favoritesCheckbox.setEnabled(false)
		else
			@favoritesCheckbox.setEnabled(true)
		end
		titlesOrNot(self)
		@multicamRadioButtons.setEnabled(true)
		# need to have both media and keywords to save XML
		if @assets.nil? || @assets.empty? || @metadata_logs.nil? || @metadata_logs.empty?
			@sendToNLEButton.setEnabled(false)
			@sendToBuilderButton.setEnabled(false)
		else
			@sendToNLEButton.setEnabled(true)
			@sendToBuilderButton.setEnabled(true)
		end
	end
	
	def titlesOrNot(sender)
		if @peopleActivitiesCheckbox.isEnabled && (@peopleActivitiesCheckbox.state==1 || @peopleKeywordsCheckbox.state==1 || @activitesCheckbox.state==1 || @keywordsCheckbox.state==1 || @favoritesCheckbox.state==1)
			@titlesCheckbox.setEnabled(true)
		elsif @peopleTranscriptsCheckbox.isEnabled && @peopleTranscriptsCheckbox.state==1
			@titlesCheckbox.setEnabled(true)
		else
			@titlesCheckbox.setState(0)
			@titlesCheckbox.setEnabled(false)
		end
	end
	ib_action :titlesOrNot
	
	def chooseMediaAdjustment(sender)
		day = @dayPopup.titleOfSelectedItem.to_i
		hrs = @hrsPopup.titleOfSelectedItem.to_i
		min = @minPopup.titleOfSelectedItem.to_i
		sec = @secPopup.titleOfSelectedItem.to_i
		if @posNegPopup.titleOfSelectedItem=="+"
			adj_time = day*24*60*60 + hrs*60*60 + min*60 + sec # seconds
		else
			adj_time = 0 - (day*24*60*60 + hrs*60*60 + min*60 + sec)
		end
		@clips = [] if @clips.nil?
		@clips.each do |clip|
			start_time = clip[:created_at] + adj_time
			stop_time = start_time + clip[:duration]
			clip[:start_time] = start_time
			clip[:stop_time] = stop_time
			clip["local_start_time"] = NSDateFormatter.localizedStringFromDate_dateStyle_timeStyle(start_time.to_ns, NSDateFormatterShortStyle, NSDateFormatterMediumStyle)
			clip["local_stop_time"] = NSDateFormatter.localizedStringFromDate_dateStyle_timeStyle(stop_time.to_ns, NSDateFormatterShortStyle, NSDateFormatterMediumStyle)
			unless clip[:timebase].nil?
				adj_frames = to_frames(adj_time.to_s, (1 / clip[:timebase]).to_s, clip[:tcFormat])
				clip[:start_frame] = clip[:first_frame] + adj_frames
				clip[:stop_frame] = clip[:first_frame] + clip[:duration_frames] + adj_frames
				clip["start_timecode"] = to_timecode(clip[:start_frame], clip[:timebase], clip[:tcFormat])
				clip["stop_timecode"] = to_timecode(clip[:stop_frame], clip[:timebase], clip[:tcFormat])
			end
		end
		@mediaTableView.reloadData
	end
	ib_action :chooseMediaAdjustment
	
	def chooseTranscriptsFolder(sender)
		#puts "chooseTranscriptsFolder", sender.inspect
		oPanel = NSOpenPanel.openPanel
		oPanel.setMessage("Choose a folder containing transcriptions as .txt, .rtf, .rtfd, .doc or .docx documents")
		oPanel.setCanChooseFiles(false)
		oPanel.setCanChooseDirectories(true)
		oPanel.setAllowsMultipleSelection(false)
		oPanel.beginSheetForDirectory_file_types_modalForWindow_modalDelegate_didEndSelector_contextInfo(nil, nil, nil, @mainWindow, self, 'didEndChooseFolder:returnCode:contextInfo:', nil)
	end
	ib_action :chooseTranscriptsFolder
	
	def didEndChooseFolder_returnCode_contextInfo(oPanel, buttonClicked, info)
		if buttonClicked==1
			oPanel.orderOut(self)
			transcripts_folder = oPanel.URL.path.to_s
			@spinner.startAnimation(self)
			@eventPopup.selectItemAtIndex(0)
			@people, @activities, @keywords, @favorites = Array.new, Array.new, Array.new, Array.new
			@metadata_logs = Array.new
			if @nokogiri_xml.nil?
				# re-parse event XML
				@nokogiri_xml = Nokogiri.XML(@xml_string) { |config| config.noblanks }
			end
			#puts "begin"
			begin
				### dryrun check that timestamps in all transcripts progress as expected
				transcript_problems = Array.new
				can_continue = true
				# @event_clips_nodeset = @nokogiri_xml.xpath(Event_MCclip_XPath) + @nokogiri_xml.xpath(Event_Clip_XPath) # process mc-clip before others so we can determine if an asset is part of a multicam (and its clip can be ignored for string-outs)
				@event_clips_array = @event_clips_nodeset.dup.to_a
				#puts "#{@event_clips_array.length} @event_clips_array"
				@event_clips_array.each_with_index do |clip_node, index|
					#puts "#{index} Is #{clip_node["name"]} nil? #{clip_node.nil?}"
					unless clip_node.nil?
						problems, continue, array_of_speakers = parse_transcript_for_clip(clip_node, transcripts_folder, true) # dryrun = true
						#puts "problems #{problems}", "continue? #{continue}"
						can_continue = false if continue==false
						if problems.nil?
							#puts "no problems with timestamps for #{clip_node["name"]}"
							@people += array_of_speakers.uniq
						else
							transcript_problems << problems
							@event_clips_array[index] = nil # remove clips we won't be working on
						end
					end
				end
				#puts "can_continue? #{can_continue}"
				@event_clips_array.compact!
				#puts "#{@event_clips_array.length} @event_clips_array"
				if transcript_problems.empty?
					#puts "no transcript_problems"
				else
					report = transcript_problems.join("\n")
					#puts "\nTRANSCRIPT PROBLEMS:\n", report
					@spinner.stopAnimation(self)
					alert = NSAlert.alloc.init
					alert.setMessageText("There was a problem with the transcripts")
					alert.setInformativeText(report)
					alert.setAlertStyle(NSCriticalAlertStyle)
					alert.beginSheetModalForWindow_modalDelegate_didEndSelector_contextInfo(@mainWindow, self, nil, nil) # Deprecated after 10.10
					if can_continue
						# missing transcripts only
					else
						return
					end
				end
				@people.uniq!
				### get event clip/mc-clip and find transcripts for the clip
				@transcript_logs = Array.new # need to hold these so we can add filtered by relevance @extracted_logs
				text_list = Array.new # an array of full text transcripts (one for each clip) for keyword extraction
				@event_clips_array.each_with_index do |clip_node, index|
					#puts "---------------------------------------", clip_node["name"]
					arrays_of_words, array_of_speakers, arrays_of_timestamps = parse_transcript_for_clip(clip_node, transcripts_folder)
					keyword_array = get_transcript_keywords_for_clip(arrays_of_words, array_of_speakers, arrays_of_timestamps, clip_node, index)
					clip_transcript = Array.new
					keyword_array.each do |k|
						clip_transcript << k[:transcript]
					end
					text_list << clip_transcript.join(" ")
					array_for_builder = false
					# if $textmate && array_for_builder
					# 	#puts "call to watson..."
					# 	transcript_text = clip_transcript.join(" ").gsub("&apos;", "'").gsub("&lt;", "<").gsub("&gt;", ">") # unescape the previously escaped characters for the NSTableView OR Watson
					# 	watson_concepts, watson_entities, watson_keywords = get_watson_arrays(transcript_text)
					# 	#puts "...and back"
					# 	keyword_array.each do |k|
					# 		k[:watson_concepts] = watson_concepts
					# 		k[:watson_entities] = Array.new
					# 		watson_entities.each do |w|
					# 			if k[:transcript].include?(w.first)
					# 				k[:watson_entities] << w
					# 			end
					# 		end
					# 		k[:watson_keywords] = Array.new
					# 		watson_keywords.each do |w|
					# 			if k[:transcript].include?(w.first)
					# 				k[:watson_keywords] << w
					# 			end
					# 		end
					# 	end
					# end # if $textmate
					@transcript_logs += keyword_array
				end
				@extracted_logs = Array.new # need to hold these so we can filter by relevance and add to @transcript_logs
				all_extracted_keywords = Array.new # the extracted keywords from all transcripts sent to monkeylearn
				if $localhost
					print "no monkeylearn!!!1!\n"
					results = Array.new
					@event_clips_array.each { |c| results << [] }
				else
					results = get_keyword_extractor_array(text_list) # one result for each item in the text_list array
				end
				#pp results
				min_relevance = 20 / 100.0
				@event_clips_array.each_with_index do |clip_node, index|
					#puts "---------------------------------------", clip_node["name"]
					extracted_keywords_for_clip = results[index].find_all { |k| k["relevance"].to_f >= min_relevance } # API returns 50 keywords, but only work with more relevant
					#puts "#{extracted_keywords_for_clip.length} Magic Keywords"
					all_extracted_keywords += extracted_keywords_for_clip
					transcript_paragraphs_for_clip = @transcript_logs.find_all { |l| l[:clip_node_index]==index }
					transcript_paragraphs_for_clip.each do |paragraph|
						extracted_keywords_for_clip.each do |keyword|
							metadata = keyword["keyword"]
							relevance = keyword["relevance"].to_f
							if paragraph[:transcript].include?(metadata)
								keyword_hash = { :clip_node => paragraph[:clip_node], :clip_node_index => paragraph[:clip_node_index], :format_id => paragraph[:format_id], :frameDuration => paragraph[:frameDuration], :tcFormat => paragraph[:tcFormat], :speaker => paragraph[:speaker], :name => metadata, "metadata" => metadata, :relevance => relevance, :checkbox => 1, :keyword_first_frame => paragraph[:keyword_first_frame]+1, :keyword_duration => paragraph[:keyword_duration]-1, :start_time => paragraph[:start_time], :stop_time => paragraph[:stop_time] - paragraph[:frameDuration], "start_timecode" => paragraph["start_timecode"], "stop_timecode" => paragraph["stop_timecode"] } # start keyword 1 frame after paragraph to sort properly; reduce duration by 1 frame too
								# REMINDER:
								# metadata = raw_metadata.uniq.sort_by { |m| m[:start_time] }
								# keywords << { :name => m["metadata"], :start_time => m[:start_time], :stop_time => m[:stop_time], :person => person, :stars => stars }
								@extracted_logs << keyword_hash
							end
						end
					end
				end
				#puts "#{@extracted_logs.length} extracted keywords"
				if @extracted_logs.empty?
					@metadata_logs = @transcript_logs
				else
					# build the labels for the slider ticks
					# @firstMagicKeywordsLabel.setStringValue("none")
					# second_relevance = all_extracted_keywords.find_all { |k| k["relevance"].to_f >= 80/100.0 }.length
					# @secondMagicKeywordsLabel.setStringValue(second_relevance)
					# third_relevance = all_extracted_keywords.find_all { |k| k["relevance"].to_f >= 60/100.0 }.length
					# @thirdMagicKeywordsLabel.setStringValue(third_relevance)
					# fourth_relevance = all_extracted_keywords.find_all { |k| k["relevance"].to_f >= 40/100.0 }.length
					# @fourthMagicKeywordsLabel.setStringValue(fourth_relevance)
					# fifth_relevance = all_extracted_keywords.find_all { |k| k["relevance"].to_f >= 20/100.0 }.length
					# @fifthMagicKeywordsLabel.setStringValue(fifth_relevance)
					# filterMagicKeywordsByRelevance(self)
				end
				if @metadata_logs.empty?
					#puts "**** No logs extracted from transcripts"
				else
					@mode = "Transcript"
					@mainWindow.setTitle("Lumberyard — Transcript Mode")
					@mediaTableView.tableColumns[0].headerCell.setStringValue("Clips")
					@mediaTableView.tableColumns[1].headerCell.setStringValue("Start Timecode")
					@mediaTableView.tableColumns[2].headerCell.setStringValue("Stop Timecode")
					@keywordsTableView.tableColumns[0].headerCell.setStringValue("Transcript")
					@keywordsTableView.tableColumns[1].headerCell.setStringValue("Start Timecode")
					@keywordsTableView.tableColumns[2].headerCell.setStringValue("Stop Timecode")
				end
				@keywordsTableView.reloadData
				@people.uniq!
				if @people.length==1
					@peopleTranscriptsCheckbox.setTitle("each person's transcripts (1 person)")
				else
					@peopleTranscriptsCheckbox.setTitle("each person's transcripts (#{@people.length} people)")
				end
				stringOutsOrNot(self)
				@spinner.stopAnimation(self)
			rescue => e
				@spinner.stopAnimation(self)
				if $textmate
					raise e
				else
					alert = NSAlert.alloc.init
					alert.setMessageText("There was a problem with these transcript documents")
					alert.setInformativeText(e.message)
					alert.setAlertStyle(NSCriticalAlertStyle)
					alert.beginSheetModalForWindow_modalDelegate_didEndSelector_contextInfo(@mainWindow, self, nil, nil) # Deprecated after 10.10
				end
			end
		end
	end
	
	def filterMagicKeywordsByRelevance(sender) # triggered on moving the @magicKeywordsRelevanceSlider
		cutoff = @magicKeywordsRelevanceSlider.floatValue.abs / 100.0
		cutoff = 2 if cutoff==1.0
		magic_keywords = @extracted_logs.find_all { |r| r[:relevance] >= cutoff }
		@keywords = Array.new # array for either other keywords or magic keywords
		uniq_magic_keywords = Array.new
		magic_keywords.each do |m|
			uniq_magic_keywords << m["metadata"]
			if m[:checkbox]==1
				@keywords << m["metadata"]
			else
				#puts "#{m["metadata"]} is off"
			end
		end
		@keywords.uniq! # the active keywords only
		@keywords.sort!
		unsorted_metadata_logs = @transcript_logs + magic_keywords
		@metadata_logs = unsorted_metadata_logs.sort { |m,n| m[:clip_node_index]==n[:clip_node_index]? m[:keyword_first_frame]<=>n[:keyword_first_frame] : m[:clip_node_index]<=>n[:clip_node_index] } # sort by :clip_node_index then by :keyword_first_frame
		stringOutsOrNot(self) # update the checkbox labels
		@keywordsTableView.reloadData
	end
	ib_action :filterMagicKeywordsByRelevance
	
	def chooseCountry(sender) # triggered on changing Country timezone popup
		gem 'tzinfo'
		require 'tzinfo'
		country = @countries.find { |c| c.to_s==@tzCountryPopup.titleOfSelectedItem }
		@tzCityPopup.removeAllItems
		if country.nil?
			@country_time_zones = []
		else
			@country_time_zones = TZInfo::Country.get(country.code).zones
		end
		last_city = NSUserDefaults.standardUserDefaults.stringForKey("TzCity")
		selected_city = nil
		@country_time_zones.each do |z|
			@tzCityPopup.addItemWithTitle(z.to_s)
			selected_city = last_city if last_city==z.to_s
		end
		unless selected_city.nil?
			@tzCityPopup.selectItemWithTitle(selected_city)
		end
		chooseCity(self)
	end
	ib_action :chooseCountry
	
	def chooseCity(sender) # triggered on changing the Country or City timezone popups
		gem 'tzinfo'
		require 'tzinfo'
		camera_zone = @country_time_zones.find { |c| c.to_s==@tzCityPopup.titleOfSelectedItem }
		@camera_timezone = TZInfo::Timezone.get(camera_zone.identifier)
		@server_timezone = TZInfo::Timezone.get("America/Los_Angeles")
		@timezone_adj = 0 # was (server_time.now - camera_time.now).round # calculate time adjustment for selected timezone
	end
	ib_action :chooseCity
	
	def clickSendToNLE(sender)
		@target_app_identifier = @NLE_identifier.to_s # @NLE_identifier can be nil
		if @XML_flavor=="fcpxml"
			save_event_fcpxml()
		elsif @XML_flavor=="xmeml"
			save_project_xmeml()
		end
	end
	ib_action :clickSendToNLE
	
	def clickSendToBuilder(sender)
		@target_app_identifier = @BLDR_identifier
		if @XML_flavor=="fcpxml"
			save_event_fcpxml()
		elsif @XML_flavor=="xmeml"
			save_project_xmeml()
		end
	end
	ib_action :clickSendToBuilder
	
	def save_event_fcpxml()
		@sendToNLEButton.highlight(false)
		@spinner.startAnimation(self)
		@progressIndicator.setDoubleValue(0)
		@progressIndicator.setIndeterminate(false)
		@progressMessage.setStringValue("Working...\n")
		@progressView.displayIfNeeded
		NSApp.beginSheet_modalForWindow_modalDelegate_didEndSelector_contextInfo(@progressPanel, @mainWindow, self, nil, nil)
		begin
			@added_metadata = 0
			@onto_clips = 0
			@added_stringouts = 0
			if @mode=="Transcript"
				add_transcripts_to_clips()
				save_fcpxml()
			else
				add_logs_to_fcpxml_clips()
				save_fcpxml()
			end
			@new_xml = @new_event_fcpxml
			didEndProgressSheet_returnCode_contextInfo(@progressPanel, 1, nil)
		rescue => e
			if $textmate
				raise e
			else
				didEndProgressSheet_returnCode_contextInfo(@progressPanel, -1, nil)
				alert = NSAlert.alloc.init
				alert.setMessageText("There was a problem adding #{@keywordORmarker}s to this Event XML file")
				alert.setInformativeText(e.message)
				alert.setAlertStyle(NSCriticalAlertStyle)
				alert.beginSheetModalForWindow_modalDelegate_didEndSelector_contextInfo(@mainWindow, self, nil, nil) # Deprecated after 10.10
			end
		end
	end
	
	def save_project_xmeml()
		@sendToNLEButton.highlight(false)
		@spinner.startAnimation(self)
		@progressIndicator.setDoubleValue(0)
		@progressIndicator.setIndeterminate(false)
		@progressMessage.setStringValue("Working...\n")
		@progressView.displayIfNeeded
		NSApp.beginSheet_modalForWindow_modalDelegate_didEndSelector_contextInfo(@progressPanel, @mainWindow, self, nil, nil)
		begin
			@added_metadata = 0
			@onto_clips = 0
			@added_stringouts = 0
			add_logs_to_xmeml_clips()
			save_xmeml()
			@new_xml = @new_project_xmeml
			didEndProgressSheet_returnCode_contextInfo(@progressPanel, 2, nil)
		rescue => e
			if $textmate
				raise e
			else
				didEndProgressSheet_returnCode_contextInfo(@progressPanel, -1, nil)
				alert = NSAlert.alloc.init
				alert.setMessageText("There was a problem adding #{@keywordORmarker}s to this Event XML file")
				alert.setInformativeText(e.message)
				alert.setAlertStyle(NSCriticalAlertStyle)
				alert.beginSheetModalForWindow_modalDelegate_didEndSelector_contextInfo(@mainWindow, self, nil, nil) # Deprecated after 10.10
			end
		end
	end
	
	def didEndProgressSheet_returnCode_contextInfo(sheet, code, info)
		#puts "didEndProgressSheet_returnCode_contextInfo code #{code}"
		@progressMessage.setStringValue("")
		@spinner.stopAnimation(self)
		@mainWindow.endSheet(@progressPanel) # NSApp.endSheet(@progressPanel)
		@progressPanel.orderOut(self)
		if code>0
			alert = NSAlert.alloc.init
			if @added_metadata==0 && @events_already_logged==false
				alert.setMessageText("No logged metadata was found for media in this Event - perhaps this is the wrong Event or time adjustment?")
				alert.setAlertStyle(NSCriticalAlertStyle)
				alert.beginSheetModalForWindow_modalDelegate_didEndSelector_contextInfo(@mainWindow, self, nil, nil) # Deprecated after 10.10
			else
				if @events_already_logged
					alert.setMessageText("Created #{@added_stringouts} selects string-outs")
				elsif @added_stringouts > 0
					alert.setMessageText("Added #{@added_metadata} #{@keywordORmarker}s to #{@onto_clips} clips and created #{@added_stringouts} selects string-outs")
				elsif @has_project_mcclips
					alert.setMessageText("Added #{@added_metadata} #{@keywordORmarker}s to #{@onto_clips} clips in this Event and Project")
				else
					alert.setMessageText("Added #{@added_metadata} #{@keywordORmarker}s to #{@onto_clips} clips in this Event")
				end
				if $textmate
					alert.addButtonWithTitle("Send to TextMate")
				elsif @target_app_identifier.include?("com.apple.FinalCut")
					alert.setInformativeText("In Final Cut Pro X, click the Replace button to have your clips updated with keywords and ratings")
					alert.addButtonWithTitle("Send to Final Cut Pro X")
				elsif @target_app_identifier==@BLDR_identifier
					alert.setInformativeText("Builder will ignore any selects string-outs in the XML")
					alert.addButtonWithTitle("Send to Builder")
				else # PPro or "" if PPro is not running
					alert.addButtonWithTitle("Send to Premiere Pro")
				end
				alert.beginSheetModalForWindow_modalDelegate_didEndSelector_contextInfo(@mainWindow, self, 'didEndAlertSheet:returnCode:contextInfo:', nil)
			end
		end
	end
	
	def didEndAlertSheet_returnCode_contextInfo(sheet, buttonClicked, info)
		#puts "didEndAlertSheet_returnCode_contextInfo buttonClicked #{buttonClicked} info #{info}"
		sheet.window.orderOut(self)
		if $textmate
			NSWorkspace.sharedWorkspace.openFile(@text_report)
			NSWorkspace.sharedWorkspace.openFile_withApplication(@new_xml, "/Applications/TextMate.app")
		else
			app_url = NSWorkspace.sharedWorkspace.URLForApplicationWithBundleIdentifier(@target_app_identifier)
			if app_url.nil?
				alert = NSAlert.alloc.init
				if @target_app_identifier.include?("com.apple.FinalCut")
					app = "Final Cut Pro X"
					alert.setMessageText("#{app} is not installed on this Mac")
				elsif @target_app_identifier==@BLDR_identifier
					app = "Builder"
					alert.setMessageText("#{app} is not installed on this Mac")
				else # PPro or "" if PPro is not running
					app = "Premiere Pro"
					alert.setMessageText("#{app} is not running or \nis not installed on this Mac")
				end
				alert.setInformativeText("Unable to send XML to #{app}")
				alert.setAlertStyle(NSCriticalAlertStyle)
				alert.beginSheetModalForWindow_modalDelegate_didEndSelector_contextInfo(@mainWindow, self, nil, nil) # Deprecated after 10.10
			else
				NSWorkspace.sharedWorkspace.openFile_withApplication(@new_xml, app_url.path)
			end
		end
	end
	
	def add_fcpxml_stringouts(stringout_clips)
		#puts "add_fcpxml_stringouts"
		### CREATE STRING OUT COMPOUND CLIPS ###
		#puts "### CREATE STRING OUT COMPOUND CLIPS ### for #{stringout_clips.length}"
		#pp @people
		added_stringouts = 0
		#puts "----------", "#{stringout_clips.first[:keywords].length} keywords"
		#puts "----------", "#{stringout_clips.first[:activities].length} activities", "----------"
		frameDuration = stringout_clips.last[:frameDuration]
		one_second = stringout_clips.last[:one_second]
		format = stringout_clips.last[:format]
		tcFormat = stringout_clips.last[:tcFormat]
		# NOTE activities << { :name => m["metadata"], :start => start, :duration => duration, :person => person, :stars => stars }
		# NOTE keywords << { :name => m["metadata"], :start => start, :duration => duration }
		titles = Array.new
		type = nil
		@people.each { |p| titles << "#{p} - Person" }
		titles.uniq!
		resources_node = @nokogiri_xml.at_xpath(".//resources")
		if $int_dtd==3
			event_node = @nokogiri_xml.at_xpath("fcpxml/project")
		else
			event_node = @nokogiri_xml.at_xpath(".//event")
		end
		resources_node.add_child('<effect id="t1" name="Basic Lower Third" uid=".../Titles.localized/Lower Thirds.localized/Basic Lower Third.localized/Basic Lower Third.moti"/>') if resources_node.at_xpath("effect[@id='t1']").nil?
		resources_node.add_child('<effect id="t2" name="Cross Dissolve" uid="FxPlug:4731E73A-8DAC-4113-9A30-AE85B1761265"/>') if resources_node.at_xpath("effect[@id='t2']").nil?
		# builder XML
		# NOTE stringout_clips << { :multicam => true/false, :name => metadata_clip_name, :ref => clip_node["ref"], :format => format_id, :first_frame_rational => to_rational(clip_node["start"]), :tcFormat => tcFormat, :frameDuration => frameDuration, :one_second => one_second.round, :start_time => clip_start_time, :stop_time => clip_stop_time, :video_angles => video_angles, :audio_angle => audio_angle, :metadata => this_clips_metadata, :activities => activities.sort_by { |a| a[:start_time] }, :keywords => keywords.sort_by { |k| k[:start_time] }, :favorites => favorites, :keywords_xml => keywords_xml }
		# for each person's activities
		if @peopleActivitiesCheckbox.state==1
			@people.each_with_index do |person, i|
				#puts "#{i} #{person.inspect}"
				total_duration, added_clips = 0, 0
				builder = Nokogiri::XML::Builder.new do |cc|
					titles_clone = titles.dup
					cc.media(:id => "pa#{i+1}", :name => "#{person} activities") {
						cc.sequence(:format => format, :tcFormat => tcFormat, :tcStart => "0s") {
							cc.spine {
								duration = 2 * one_second * frameDuration # Integer * Integer * Rational
								# cc.gap(:duration => duration)
								total_duration += duration
								angle_index = -1
								stringout_clips.each do |clip|
									clip[:activities].each do |log|
										#puts "log = #{log.inspect}"
										if log[:person]=="#{person} - Person"
											add_this_clip = true
											added_clips += 1
										else
											add_this_clip = false
										end
										#puts "add_this_clip? #{add_this_clip}"
										if add_this_clip
											start = clip[:first_frame_rational] + ((log[:start_time] - clip[:start_time]).round * one_second * frameDuration) # Integer * Integer * Rational
											start = 0 if start < 0
											duration = (log[:stop_time] - log[:start_time]).round * one_second * frameDuration
											if clip[:multicam]
												angle_index += 1
												angle_index = 0 if clip[:video_angles][angle_index].nil? || @multicamRadioButtons.selectedCell.tag==1 # reset to first
												cc.send("mc-clip", :name => clip[:name], :ref => clip[:ref], :offset => total_duration, :start => start, :duration => duration) {
													cc.send("mc-source", :angleID => clip[:video_angles][angle_index], :srcEnable => "video")
													cc.send("mc-source", :angleID => clip[:audio_angle], :srcEnable => "audio")
													if titles_clone.include?(log[:person]) && @titlesCheckbox.state==1
														#puts "add lower third for #{log[:person]}"
														cc.spine(:lane => "1", :offset => start) {
															titles_clone.delete(log[:person])
															cc.transition(:offset => 0, :name => "Cross Dissolve", :duration => one_second * frameDuration) {
																cc.send("filter-video", :ref => "t2")
															}
															cc.title(:ref => "t1", :offset => 0, :start => "3600s", :duration => 4 * one_second * frameDuration, :name => "Lower Third: #{log[:person].sub(" - Person", "")}", :role => "titles.Lumberjack") {
																cc << "<text>#{log[:person].sub(" - Person", "")}</text>"
																# can't use cc.text
																cc << "<text/>"
															}
															cc.transition(:offset => 3 * one_second * frameDuration, :name => "Cross Dissolve", :duration => one_second * frameDuration) {
																cc.send("filter-video", :ref => "t2")
															}
														}
													end
													cc << clip[:keywords_xml] # append keywords as string
												}
											else # not multicam
												cliptype = cc.send(clip[:type], :ref => clip[:ref], :name => clip[:name], :offset => total_duration, :start => start, :duration => duration, :tcFormat => clip[:tcFormat]) {
													cc << clip[:child_nodes1]
													if titles_clone.include?(log[:person]) && @titlesCheckbox.state==1
														#puts "add lower third for #{log[:person]}"
														cc.spine(:lane => "1", :offset => start) {
															titles_clone.delete(log[:person])
															cc.transition(:offset => 0, :name => "Cross Dissolve", :duration => one_second * frameDuration) {
																cc.send("filter-video", :ref => "t2")
															}
															cc.title(:ref => "t1", :offset => 0, :start => "3600s", :duration => 4 * one_second * frameDuration, :name => "Lower Third: #{log[:person].sub(" - Person", "")}", :role => "titles.Lumberjack") {
																cc << "<text>#{log[:person].sub(" - Person", "")}</text>"
																# can't use cc.text
																cc << "<text/>"
															}
															cc.transition(:offset => 3 * one_second * frameDuration, :name => "Cross Dissolve", :duration => one_second * frameDuration) {
																cc.send("filter-video", :ref => "t2")
															}
														}
													end
													cc << clip[:keywords_xml] # append keywords as string
													cc << clip[:child_nodes2]
												}
											end
											#puts "total_duration = #{total_duration} + #{duration}"
											total_duration += duration
											#puts "total_duration #{total_duration}"
										end
									end
								end
							}
						}
					}
				end
				if total_duration==2 * one_second * frameDuration
					@report << "No activities for Person: #{person}\n"
					@report << "--------------------------------------------\n"
				else
					if total_duration < 0
						total_duration = total_duration.abs
						#puts "total_duration #{total_duration}"
					end
					if added_clips < 3
						@report << "Not enough activites for Person: #{person}\n"
						@report << "--------------------------------------------\n"
					else
						@report << "Creating string-out for #{person} activities\n"
						@report << "--------------------------------------------\n"
						builder.doc.xpath("//clip | //sync-clip").each { |c| c.delete("ref") }
						media = builder.doc.at_xpath("media")
						resources_node.add_child(media)
						event_node.add_child(Nokogiri::XML.fragment("<ref-clip ref='pa#{i+1}' name='#{person.gsub("'", "&apos;")} activities' duration='#{total_duration}'><keyword value='String-outs' duration='#{total_duration}'/></ref-clip>")) # the reference in the clip area
						added_stringouts += 1
					end
				end
			end # @people.each
		end # if @peopleActivitesCheckbox
		# for each person's keywords
		if @peopleKeywordsCheckbox.state==1
			@people.each_with_index do |person, i|
				#puts "#{i} #{person.inspect}"
				total_duration, added_clips = 0, 0
				builder = Nokogiri::XML::Builder.new do |cc|
					titles_clone = titles.dup
					cc.media(:id => "pk#{i+1}", :name => "#{person} keywords") {
						cc.sequence(:format => format, :tcFormat => tcFormat, :tcStart => "0s") {
							cc.spine {
								duration = 2 * one_second * frameDuration # Integer * Integer * Rational
								# cc.gap(:duration => duration)
								total_duration += duration
								angle_index = -1
								stringout_clips.each do |clip|
									clip[:keywords].each do |log|
										#pp log
										if log[:person]=="#{person} - Person"
											add_this_clip = true
											added_clips += 1
										else
											add_this_clip = false
										end
										#puts "add_this_clip? #{add_this_clip}"
										if add_this_clip
											start = clip[:first_frame_rational] + ((log[:start_time] - clip[:start_time]).round * one_second * frameDuration) # Integer * Integer * Rational
											start = 0 if start < 0
											duration = (log[:stop_time] - log[:start_time]).round * one_second * frameDuration
											if clip[:multicam]
												angle_index += 1
												angle_index = 0 if clip[:video_angles][angle_index].nil? || @multicamRadioButtons.selectedCell.tag==1 # reset to first
												cc.send("mc-clip", :name => clip[:name], :ref => clip[:ref], :offset => total_duration, :start => start, :duration => duration) {
													cc.send("mc-source", :angleID => clip[:video_angles][angle_index], :srcEnable => "video")
													cc.send("mc-source", :angleID => clip[:audio_angle], :srcEnable => "audio")
													if titles_clone.include?(log[:person]) && @titlesCheckbox.state==1
														#puts "add lower third for #{log[:person]}"
														cc.spine(:lane => "1", :offset => start) {
															titles_clone.delete(log[:person])
															cc.transition(:offset => 0, :name => "Cross Dissolve", :duration => one_second * frameDuration) {
																cc.send("filter-video", :ref => "t2")
															}
															cc.title(:ref => "t1", :offset => 0, :start => "3600s", :duration => 4 * one_second * frameDuration, :name => "Lower Third: #{log[:person].sub(" - Person", "")}", :role => "titles.Lumberjack") {
																cc << "<text>#{log[:person].sub(" - Person", "")}</text>"
																# can't use cc.text
																cc << "<text/>"
															}
															cc.transition(:offset => 3 * one_second * frameDuration, :name => "Cross Dissolve", :duration => one_second * frameDuration) {
																cc.send("filter-video", :ref => "t2")
															}
														}
													end
													cc << clip[:keywords_xml] # append keywords as string
												}
											else # not multicam
												cc.send(clip[:type], :ref => clip[:ref], :name => clip[:name], :offset => total_duration, :start => start, :duration => duration, :tcFormat => clip[:tcFormat]) {
													cc << clip[:child_nodes1]
													if titles_clone.include?(log[:person]) && @titlesCheckbox.state==1
														#puts "add lower third for #{log[:person]}"
														cc.spine(:lane => "1", :offset => start) {
															titles_clone.delete(log[:person])
															cc.transition(:offset => 0, :name => "Cross Dissolve", :duration => one_second * frameDuration) {
																cc.send("filter-video", :ref => "t2")
															}
															cc.title(:ref => "t1", :offset => 0, :start => "3600s", :duration => 4 * one_second * frameDuration, :name => "Lower Third: #{log[:person].sub(" - Person", "")}", :role => "titles.Lumberjack") {
																cc << "<text>#{log[:person].sub(" - Person", "")}</text>"
																# can't use cc.text
																cc << "<text/>"
															}
															cc.transition(:offset => 3 * one_second * frameDuration, :name => "Cross Dissolve", :duration => one_second * frameDuration) {
																cc.send("filter-video", :ref => "t2")
															}
														}
													end
													cc << clip[:keywords_xml] # append keywords as string
													cc << clip[:child_nodes2]
												}
											end
											total_duration += duration
										end
									end
								end
							}
						}
					}
				end
				if total_duration==2 * one_second * frameDuration
					@report << "No other keywords for Person: #{person}\n"
					@report << "--------------------------------------------\n"
				else
					if total_duration < 0
						total_duration = total_duration.abs
						#puts "total_duration #{total_duration}"
					end
					if added_clips < 3
						@report << "Not enough other keywords for Person: #{person}\n"
						@report << "--------------------------------------------\n"
					else
						@report << "Creating string-out for #{person} keywords\n"
						@report << "--------------------------------------------\n"
						builder.doc.xpath("//clip | //sync-clip").each { |c| c.delete("ref") }
						media = builder.doc.at_xpath("media")
						resources_node.add_child(media)
						event_node.add_child(Nokogiri::XML.fragment("<ref-clip ref='pk#{i+1}' name='#{person.gsub("'", "&apos;")} keywords' duration='#{total_duration}'><keyword value='String-outs' duration='#{total_duration}'/></ref-clip>")) # the reference in the clip area
						added_stringouts += 1
					end
				end
			end # @people.each
		end # if @peopleKeywordsCheckbox
		# for each person's transcripts
		if @peopleTranscriptsCheckbox.state==1
			@people.each_with_index do |person, i|
				#puts "#{i} #{person.inspect}"
				total_duration, added_clips = 0, 0
				builder = Nokogiri::XML::Builder.new do |cc|
					titles_clone = titles.dup
					cc.media(:id => "pt#{i+1}", :name => "#{person} transcripts") {
						cc.sequence(:format => format, :tcFormat => tcFormat, :tcStart => "0s") {
							cc.spine {
								duration = 2 * one_second * frameDuration # Integer * Integer * Rational
								# cc.gap(:duration => duration)
								total_duration += duration
								angle_index = -1
								stringout_clips.each do |clip|
									clip[:keywords].each do |log|
										#pp log
										if log[:person]=="#{person} - Person"
											add_this_clip = true
											added_clips += 1
										else
											add_this_clip = false
										end
										#puts "add_this_clip? #{add_this_clip}"
										if add_this_clip
											start = clip[:first_frame_rational] + ((log[:start_time] - clip[:start_time]).round * one_second * frameDuration) # Integer * Integer * Rational
											start = 0 if start < 0
											duration = (log[:stop_time] - log[:start_time]).round * one_second * frameDuration
											if clip[:multicam]
												angle_index += 1
												angle_index = 0 if clip[:video_angles][angle_index].nil? || @multicamRadioButtons.selectedCell.tag==1 # reset to first
												cc.send("mc-clip", :name => clip[:name], :ref => clip[:ref], :offset => total_duration, :start => start, :duration => duration) {
													cc.send("mc-source", :angleID => clip[:video_angles][angle_index], :srcEnable => "video")
													cc.send("mc-source", :angleID => clip[:audio_angle], :srcEnable => "audio")
													if titles_clone.include?(log[:person]) && @titlesCheckbox.state==1
														#puts "add lower third for #{log[:person]}"
														cc.spine(:lane => "1", :offset => start) {
															titles_clone.delete(log[:person])
															cc.transition(:offset => 0, :name => "Cross Dissolve", :duration => one_second * frameDuration) {
																cc.send("filter-video", :ref => "t2")
															}
															cc.title(:ref => "t1", :offset => 0, :start => "3600s", :duration => 4 * one_second * frameDuration, :name => "Lower Third: #{log[:person].sub(" - Person", "")}", :role => "titles.Lumberjack") {
																cc << "<text>#{log[:person].sub(" - Person", "")}</text>"
																# can't use cc.text
																cc << "<text/>"
															}
															cc.transition(:offset => 3 * one_second * frameDuration, :name => "Cross Dissolve", :duration => one_second * frameDuration) {
																cc.send("filter-video", :ref => "t2")
															}
														}
													end
													cc << clip[:keywords_xml] # append keywords as string
												}
											else # not multicam
												cc.send(clip[:type], :ref => clip[:ref], :name => clip[:name], :offset => total_duration, :start => start, :duration => duration, :tcFormat => clip[:tcFormat]) {
													cc << clip[:child_nodes1]
													if titles_clone.include?(log[:person]) && @titlesCheckbox.state==1
														#puts "add lower third for #{log[:person]}"
														cc.spine(:lane => "1", :offset => start) {
															titles_clone.delete(log[:person])
															cc.transition(:offset => 0, :name => "Cross Dissolve", :duration => one_second * frameDuration) {
																cc.send("filter-video", :ref => "t2")
															}
															cc.title(:ref => "t1", :offset => 0, :start => "3600s", :duration => 4 * one_second * frameDuration, :name => "Lower Third: #{log[:person].sub(" - Person", "")}", :role => "titles.Lumberjack") {
																cc << "<text>#{log[:person].sub(" - Person", "")}</text>"
																# can't use cc.text
																cc << "<text/>"
															}
															cc.transition(:offset => 3 * one_second * frameDuration, :name => "Cross Dissolve", :duration => one_second * frameDuration) {
																cc.send("filter-video", :ref => "t2")
															}
														}
													end
													cc << clip[:keywords_xml] # append keywords as string
													cc << clip[:child_nodes2]
												}
											end
											total_duration += duration
										end
									end
								end
							}
						}
					}
				end
				if total_duration==2 * one_second * frameDuration
					@report << "No other keywords for Person: #{person}\n"
					@report << "--------------------------------------------\n"
				else
					if total_duration < 0
						total_duration = total_duration.abs
						#puts "total_duration #{total_duration}"
					end
					if added_clips < 3
						@report << "Not enough other keywords for Person: #{person}\n"
						@report << "--------------------------------------------\n"
					else
						@report << "Creating string-out for #{person} keywords\n"
						@report << "--------------------------------------------\n"
						builder.doc.xpath("//clip | //sync-clip").each { |c| c.delete("ref") }
						media = builder.doc.at_xpath("media")
						resources_node.add_child(media)
						event_node.add_child(Nokogiri::XML.fragment("<ref-clip ref='pt#{i+1}' name='#{person.gsub("'", "&apos;")} transcripts' duration='#{total_duration}'><keyword value='String-outs' duration='#{total_duration}'/></ref-clip>")) # the reference in the clip area
						added_stringouts += 1
					end
				end
			end # @people.each
		end # if @peopleTranscriptsCheckbox
		# for each activity
		if @activitesCheckbox.state==1
			@activities.each_with_index do |activity, i|
				#puts "#{i}: #{activity.inspect}"
				total_duration, added_clips = 0, 0
				builder = Nokogiri::XML::Builder.new do |cc|
					titles_clone = titles.dup
					cc.media(:id => "ak#{i+1}", :name => "Activity: #{activity}") {
						cc.sequence(:format => format, :tcFormat => tcFormat, :tcStart => "0s") {
							cc.spine {
								duration = 2 * one_second * frameDuration # Integer * Integer * Rational
								# cc.gap(:duration => duration)
								total_duration += duration
								angle_index = -1
								stringout_clips.each do |clip|
									clip[:activities].each do |log|
										#puts "log = #{log.inspect}"
										if log[:name]=="#{activity} - Activity"
											add_this_clip = true
											added_clips += 1
										else
											add_this_clip = false
										end
										#puts "add_this_clip? #{add_this_clip}"
										if add_this_clip
											start = clip[:first_frame_rational] + ((log[:start_time] - clip[:start_time]).round * one_second * frameDuration) # Integer * Integer * Rational
											start = 0 if start < 0
											duration = (log[:stop_time] - log[:start_time]).round * one_second * frameDuration
											if clip[:multicam]
												angle_index += 1
												angle_index = 0 if clip[:video_angles][angle_index].nil? || @multicamRadioButtons.selectedCell.tag==1 # reset to first
												cc.send("mc-clip", :name => clip[:name], :ref => clip[:ref], :offset => total_duration, :start => start, :duration => duration) {
													cc.send("mc-source", :angleID => clip[:video_angles][angle_index], :srcEnable => "video")
													cc.send("mc-source", :angleID => clip[:audio_angle], :srcEnable => "audio")
													if titles_clone.include?(log[:person]) && @titlesCheckbox.state==1
														#puts "add lower third for #{log[:person]}"
														cc.spine(:lane => "1", :offset => start) {
															titles_clone.delete(log[:person])
															cc.transition(:offset => 0, :name => "Cross Dissolve", :duration => one_second * frameDuration) {
																cc.send("filter-video", :ref => "t2")
															}
															cc.title(:ref => "t1", :offset => 0, :start => "3600s", :duration => 4 * one_second * frameDuration, :name => "Lower Third: #{log[:person].sub(" - Person", "")}", :role => "titles.Lumberjack") {
																cc << "<text>#{log[:person].sub(" - Person", "")}</text>"
																# can't use cc.text
																cc << "<text/>"
															}
															cc.transition(:offset => 3 * one_second * frameDuration, :name => "Cross Dissolve", :duration => one_second * frameDuration) {
																cc.send("filter-video", :ref => "t2")
															}
														}
													end
													cc << clip[:keywords_xml] # append keywords as string
												}
											else # not multicam
												cc.send(clip[:type], :ref => clip[:ref], :name => clip[:name], :offset => total_duration, :start => start, :duration => duration, :tcFormat => clip[:tcFormat]) {
													cc << clip[:child_nodes1]
													if titles_clone.include?(log[:person]) && @titlesCheckbox.state==1
														#puts "add lower third for #{log[:person]}"
														cc.spine(:lane => "1", :offset => start) {
															titles_clone.delete(log[:person])
															cc.transition(:offset => 0, :name => "Cross Dissolve", :duration => one_second * frameDuration) {
																cc.send("filter-video", :ref => "t2")
															}
															cc.title(:ref => "t1", :offset => 0, :start => "3600s", :duration => 4 * one_second * frameDuration, :name => "Lower Third: #{log[:person].sub(" - Person", "")}", :role => "titles.Lumberjack") {
																cc << "<text>#{log[:person].sub(" - Person", "")}</text>"
																# can't use cc.text
																cc << "<text/>"
															}
															cc.transition(:offset => 3 * one_second * frameDuration, :name => "Cross Dissolve", :duration => one_second * frameDuration) {
																cc.send("filter-video", :ref => "t2")
															}
														}
													end
													cc << clip[:keywords_xml] # append keywords as string
													cc << clip[:child_nodes2]
												}
											end
											total_duration += duration
										end
									end
								end
							}
						}
					}
				end
				if total_duration==2 * one_second * frameDuration
					@report << "No media for Activity: #{activity}\n"
					@report << "--------------------------------------------\n"
				else
					if total_duration < 0
						total_duration = total_duration.abs
						#puts "total_duration #{total_duration}"
					end
					if added_clips < 3
						@report << "Not enough media for Activity: #{activity}\n"
						@report << "--------------------------------------------\n"
					else
						@report << "Creating string-out for Activity: #{activity}\n"
						@report << "--------------------------------------------\n"
						builder.doc.xpath("//clip | //sync-clip").each { |c| c.delete("ref") }
						media = builder.doc.at_xpath("media")
						resources_node.add_child(media)
						event_node.add_child(Nokogiri::XML.fragment("<ref-clip ref='ak#{i+1}' name='Activity: #{activity.gsub("'", "&apos;")}' duration='#{total_duration}'><keyword value='String-outs' duration='#{total_duration}'/></ref-clip>")) # the reference in the clip area
						added_stringouts += 1
					end
				end
			end # @activites.each
		end # if @activitesCheckbox
		# for each other keyword
		if @keywordsCheckbox.state==1
			@keywords.each_with_index do |keyword, i|
				#puts "#{i} #{keyword.inspect}"
				total_duration, added_clips = 0, 0
				builder = Nokogiri::XML::Builder.new do |cc|
					titles_clone = titles.dup
					#puts titles_clone.inspect
					cc.media(:id => "ok#{i+1}", :name => "Keyword: #{keyword}") {
						cc.sequence(:format => format, :tcFormat => tcFormat, :tcStart => "0s") {
							cc.spine {
								duration = 2 * one_second * frameDuration # Integer * Integer * Rational
								# cc.gap(:duration => duration)
								total_duration += duration
								angle_index = -1
								stringout_clips.each do |clip|
									clip[:keywords].each do |log|
										#puts "log = #{log.inspect}"
										if log[:name]=="#{keyword} - Other" || log[:name]==keyword # other keywords
											add_this_clip = true
											added_clips += 1
										else
											add_this_clip = false
										end
										#puts "add_this_clip? #{add_this_clip}"
										if add_this_clip
											#puts "start = #{clip[:first_frame_rational]} + #{((log[:start_time] - clip[:start_time]).round * one_second * frameDuration)}"
											start = clip[:first_frame_rational] + ((log[:start_time] - clip[:start_time]).round * one_second * frameDuration) # Integer * Integer * Rational
											start = 0 if start < 0
											duration = (log[:stop_time] - log[:start_time]).round * one_second * frameDuration
											if clip[:multicam]
												angle_index += 1
												angle_index = 0 if clip[:video_angles][angle_index].nil? || @multicamRadioButtons.selectedCell.tag==1 # reset to first
												cc.send("mc-clip", :name => clip[:name], :ref => clip[:ref], :offset => total_duration, :start => start, :duration => duration) {
													cc.send("mc-source", :angleID => clip[:video_angles][angle_index], :srcEnable => "video")
													cc.send("mc-source", :angleID => clip[:audio_angle], :srcEnable => "audio")
													#puts log[:person].inspect
													if titles_clone.include?(log[:person]) && @titlesCheckbox.state==1
														#puts "add lower third for #{log[:person]}"
														cc.spine(:lane => "1", :offset => start) {
															titles_clone.delete(log[:person])
															cc.transition(:offset => 0, :name => "Cross Dissolve", :duration => one_second * frameDuration) {
																cc.send("filter-video", :ref => "t2")
															}
															cc.title(:ref => "t1", :offset => 0, :start => "3600s", :duration => 4 * one_second * frameDuration, :name => "Lower Third: #{log[:person].sub(" - Person", "")}", :role => "titles.Lumberjack") {
																cc << "<text>#{log[:person].sub(" - Person", "")}</text>"
																# can't use cc.text
																cc << "<text/>"
															}
															cc.transition(:offset => 3 * one_second * frameDuration, :name => "Cross Dissolve", :duration => one_second * frameDuration) {
																cc.send("filter-video", :ref => "t2")
															}
														}
													end
													cc << clip[:keywords_xml] # append keywords as string
												}
											else # not multicam
												cc.send(clip[:type], :ref => clip[:ref], :name => clip[:name], :offset => total_duration, :start => start, :duration => duration, :tcFormat => clip[:tcFormat]) {
													cc << clip[:child_nodes1]
													if titles_clone.include?(log[:person]) && @titlesCheckbox.state==1
														#puts "add lower third for #{log[:person]}"
														cc.spine(:lane => "1", :offset => start) {
															titles_clone.delete(log[:person])
															cc.transition(:offset => 0, :name => "Cross Dissolve", :duration => one_second * frameDuration) {
																cc.send("filter-video", :ref => "t2")
															}
															cc.title(:ref => "t1", :offset => 0, :start => "3600s", :duration => 4 * one_second * frameDuration, :name => "Lower Third: #{log[:person].sub(" - Person", "")}", :role => "titles.Lumberjack") {
																cc << "<text>#{log[:person].sub(" - Person", "")}</text>"
																# can't use cc.text
																cc << "<text/>"
															}
															cc.transition(:offset => 3 * one_second * frameDuration, :name => "Cross Dissolve", :duration => one_second * frameDuration) {
																cc.send("filter-video", :ref => "t2")
															}
														}
													end
													cc << clip[:keywords_xml] # append keywords as string
													cc << clip[:child_nodes2]
												}
											end
											total_duration += duration
										end
									end
								end
							}
						}
					}
				end
				if total_duration==2 * one_second * frameDuration
					@report << "No media for Keyword: #{keyword}\n"
					@report << "--------------------------------------------\n"
				else
					if total_duration < 0
						total_duration = total_duration.abs
						#puts "total_duration #{total_duration}"
					end
					if added_clips < 3
						@report << "Not enough media for Keyword: #{keyword}\n"
						@report << "--------------------------------------------\n"
					else
						@report << "Creating string-out for Keyword: #{keyword}\n"
						@report << "--------------------------------------------\n"
						builder.doc.xpath("//clip | //sync-clip").each { |c| c.delete("ref") }
						media = builder.doc.at_xpath("media")
						resources_node.add_child(media)
						event_node.add_child(Nokogiri::XML.fragment("<ref-clip ref='ok#{i+1}' name='Keyword: #{keyword.gsub("'", "&apos;")}' duration='#{total_duration}'><keyword value='String-outs' duration='#{total_duration}'/></ref-clip>")) # the reference in the clip area
						added_stringouts += 1
					end
				end
			end # @keywords.each
		end # if @keywordsCheckbox
		# for each favorite
		if @favoritesCheckbox.state==1
			total_duration, added_clips = 0, 0
			builder = Nokogiri::XML::Builder.new do |cc|
				titles_clone = titles.dup
				#puts titles_clone.inspect
				cc.media(:id => "fav1", :name => "Favorites") {
					cc.sequence(:format => format, :tcFormat => tcFormat, :tcStart => "0s") {
						cc.spine {
							duration = 2 * one_second * frameDuration # Integer * Integer * Rational
							# cc.gap(:duration => duration)
							total_duration += duration
							angle_index = -1
							stringout_clips.each do |clip|
								clip[:favorites].each do |log|
									#puts "log = #{log.inspect}"
									added_clips += 1
									start = clip[:first_frame_rational] + ((log[:start_time] - clip[:start_time]).round * one_second * frameDuration) # Integer * Integer * Rational
									start = 0 if start < 0
									duration = (log[:stop_time] - log[:start_time]).round * one_second * frameDuration
									if clip[:multicam]
										angle_index += 1
										angle_index = 0 if clip[:video_angles][angle_index].nil? || @multicamRadioButtons.selectedCell.tag==1 # reset to first
										cc.send("mc-clip", :name => clip[:name], :ref => clip[:ref], :offset => total_duration, :start => start, :duration => duration) {
											cc.send("mc-source", :angleID => clip[:video_angles][angle_index], :srcEnable => "video")
											cc.send("mc-source", :angleID => clip[:audio_angle], :srcEnable => "audio")
											#puts log[:person].inspect
											if titles_clone.include?(log[:person]) && @titlesCheckbox.state==1
												#puts "add lower third for #{log[:person]}"
												cc.spine(:lane => "1", :offset => start) {
													titles_clone.delete(log[:person])
													cc.transition(:offset => 0, :name => "Cross Dissolve", :duration => one_second * frameDuration) {
														cc.send("filter-video", :ref => "t2")
													}
													cc.title(:ref => "t1", :offset => 0, :start => "3600s", :duration => 4 * one_second * frameDuration, :name => "Lower Third: #{log[:person].sub(" - Person", "")}", :role => "titles.Lumberjack") {
														cc << "<text>#{log[:person].sub(" - Person", "")}</text>"
														# can't use cc.text
														cc << "<text/>"
													}
													cc.transition(:offset => 3 * one_second * frameDuration, :name => "Cross Dissolve", :duration => one_second * frameDuration) {
														cc.send("filter-video", :ref => "t2")
													}
												}
											end
											cc << clip[:keywords_xml] # append keywords as string
										}
									else # not multicam
										cc.send(clip[:type], :ref => clip[:ref], :name => clip[:name], :offset => total_duration, :start => start, :duration => duration, :tcFormat => clip[:tcFormat]) {
											cc << clip[:child_nodes1]
											if titles_clone.include?(log[:person]) && @titlesCheckbox.state==1
												#puts "add lower third for #{log[:person]}"
												cc.spine(:lane => "1", :offset => start) {
													titles_clone.delete(log[:person])
													cc.transition(:offset => 0, :name => "Cross Dissolve", :duration => one_second * frameDuration) {
														cc.send("filter-video", :ref => "t2")
													}
													cc.title(:ref => "t1", :offset => 0, :start => "3600s", :duration => 4 * one_second * frameDuration, :name => "Lower Third: #{log[:person].sub(" - Person", "")}", :role => "titles.Lumberjack") {
														cc << "<text>#{log[:person].sub(" - Person", "")}</text>"
														# can't use cc.text
														cc << "<text/>"
													}
													cc.transition(:offset => 3 * one_second * frameDuration, :name => "Cross Dissolve", :duration => one_second * frameDuration) {
														cc.send("filter-video", :ref => "t2")
													}
												}
											end
											cc << clip[:keywords_xml] # append keywords as string
											cc << clip[:child_nodes2]
										}
									end
									total_duration += duration
								end
							end
						}
					}
				}
			end
			if total_duration==2 * one_second * frameDuration
				@report << "No media for Favorites\n"
				@report << "--------------------------------------------\n"
			else
				if total_duration < 0
					total_duration = total_duration.abs
					#puts "total_duration #{total_duration}"
				end
				if added_clips < 3
					@report << "Not enough media for Favorites\n"
					@report << "--------------------------------------------\n"
				else
					@report << "Creating string-out for Favorites\n"
					@report << "--------------------------------------------\n"
					builder.doc.xpath("//clip | //sync-clip").each { |c| c.delete("ref") }
					media = builder.doc.at_xpath("media")
					resources_node.add_child(media)
					event_node.add_child(Nokogiri::XML.fragment("<ref-clip ref='fav1' name='Favorites' duration='#{total_duration}'><keyword value='String-outs' duration='#{total_duration}'/></ref-clip>")) # the reference in the clip area
					added_stringouts += 1
				end
			end
		end # if @favoritesCheckbox
		return added_stringouts
	end
	
	def add_xmeml_stringouts(stringout_clips)
		#puts "add_xmeml_stringouts"
		### CREATE STRING OUT SEQUENCES ###
		#puts "### CREATE STRING OUT SEQUENCES ### for #{stringout_clips.length}"
		#pp @people
		added_stringouts = 0
		#pp stringout_clips.first
		#puts "----------", "#{stringout_clips.first[:keywords].length} keywords"
		#puts "----------", "#{stringout_clips.first[:activities].length} activities", "----------"
		ntsc = stringout_clips.last[:ntsc]
		timebase = stringout_clips.last[:timebase]
		one_second = timebase
		displayformat = stringout_clips.last[:displayformat]
		width = stringout_clips.last[:width]
		height = stringout_clips.last[:height]
		samplerate = stringout_clips.last[:samplerate]
		# NOTE activities << { :name => m["metadata"], :start => start, :duration => duration, :person => person, :stars => stars }
		# NOTE keywords << { :name => m["metadata"], :start => start, :duration => duration }
		titles = Array.new
		type = nil
		@people.each { |p| titles << "#{p} - Person" }
		titles.uniq!
		children_node = @nokogiri_xml.xpath(".//children").last
		# builder XML
		# for each person's activities
		if @peopleActivitiesCheckbox.state==1
			@people.each_with_index do |person, i|
				#puts "#{i} #{person.inspect}"
				playhead, added_clips = 0, 0
				max_audio_tracks = 0
				builder = Nokogiri::XML::Builder.new do |xmeml|
					titles_clone = titles.dup
					xmeml.sequence(:id => "pa#{i+1}") {
						xmeml.name "#{person} activities"
						xmeml.rate {
							xmeml.ntsc ntsc
							xmeml.timebase timebase
						}
						xmeml.timecode {
							xmeml.rate {
								xmeml.ntsc ntsc
								xmeml.timebase timebase
							}
							xmeml.frame "0"
							xmeml.displayformat displayformat
						}
						xmeml.in "-1"
						xmeml.out "-1"
						xmeml.media {
							xmeml.video {
								xmeml.format {
									xmeml.samplecharacteristics {
										xmeml.width width
										xmeml.height height
										xmeml.rate {
											xmeml.ntsc ntsc
											xmeml.timebase timebase
										}
										xmeml.colordepth "24"
									}
								}
								xmeml.track {
									clipindex = 0
									angle_index = -1
									stringout_clips.each do |clip|
										clip[:activities].each do |log|
											#puts "log = #{log.inspect}"
											if log[:person]=="#{person} - Person"
												add_this_clip = true
												added_clips += 1
											else
												add_this_clip = false
											end
											#puts "add_this_clip? #{add_this_clip}"
											if add_this_clip
												audio_track_count = clip[:clip_node].xpath("media/audio/track/clipitem").length
												if audio_track_count > max_audio_tracks
													max_audio_tracks = audio_track_count
												end
												#puts "max_audio_tracks = #{max_audio_tracks}"
												clipitem = clip[:clip_node].at_xpath("media/video/track/clipitem").dup
												angle_index += 1
												activevideoangle_nodes = clipitem.xpath("multiclip/angle/activevideoangle") # I've added an activevideoangle node to all video angles...
												if @multicamRadioButtons.selectedCell.tag==1 || angle_index==activevideoangle_nodes.length
													angle_index = 0
												end
												activevideoangle_nodes.each_with_index do |node, i|
													if i!=angle_index
														node.remove # ...so remove the unnecessary ones
													end
												end
												file_node = clipitem.at_xpath("file")
												file_node.children.each { |c| c.remove } # remove all children to leave a file reference
												in_node = clipitem.at_xpath("in")
												if in_node!= nil
													in_node.remove
												end
												out_node = clipitem.at_xpath("out")
												if out_node!=nil
													out_node.remove
												end
												inpoint = (log[:start_time] - clip[:start_time]).round * timebase
												inpoint = 0 if inpoint < 0
												duration = (log[:stop_time] - log[:start_time]).round * timebase
												clipitem.add_child("<in>#{inpoint}</in>")
												clipitem.add_child("<out>#{inpoint+duration}</out>")
												clipitem.add_child("<start>#{playhead}</start>")
												clipitem.add_child("<end>#{playhead+duration}</end>")
												clipindex += 1
												clipitem.xpath("link/clipindex").each do |clipindex_node|
													clipindex_node.content = clipindex
												end
												xmeml << clipitem.to_s
												playhead += duration
											end
										end
									end
								}
								xmeml.track { # titles
									playhead = 0
									stringout_clips.each do |clip|
										clip[:activities].each do |log|
											#puts "log = #{log.inspect}"
											if log[:person]=="#{person} - Person"
												add_this_clip = true
												added_clips += 1
											else
												add_this_clip = false
											end
											#puts "add_this_clip? #{add_this_clip}"
											if add_this_clip
												inpoint = (log[:start_time] - clip[:start_time]).round * timebase
												inpoint = 0 if inpoint < 0
												duration = (log[:stop_time] - log[:start_time]).round * timebase
												if titles_clone.include?(log[:person]) && @titlesCheckbox.state==1
													titles_clone.delete(log[:person])
													xmeml << lower_third_xmeml(person, playhead, ntsc, timebase)
												end
												playhead += duration
											end
										end
									end
								}
							}
							xmeml.audio {
								xmeml.format {
									xmeml.samplecharacteristics {
										xmeml.depth "16"
										xmeml.samplerate samplerate
									}
								}
								max_audio_tracks.times do |i|
									xmeml.track {
										clipindex = 0
										playhead = 0 # reset for each track
										stringout_clips.each do |clip|
											clip[:activities].each do |log|
												if log[:person]=="#{person} - Person"
													add_this_clip = true
												else
													add_this_clip = false
												end
												clipitems = clip[:clip_node].xpath("media/audio/track/clipitem")
												a_clipitem = clipitems[i-1]
												if a_clipitem.nil?
													add_this_clip = false
												end
												#puts "add_this_clip? #{add_this_clip}"
												if add_this_clip
													clipitem = a_clipitem.dup
													file_node = clipitem.at_xpath("file")
													file_node.children.each { |c| c.remove } # remove all children to leave a file reference
													in_node = clipitem.at_xpath("in")
													if in_node!= nil
														in_node.remove
													end
													out_node = clipitem.at_xpath("out")
													if out_node!=nil
														out_node.remove
													end
													inpoint = (log[:start_time] - clip[:start_time]).round * timebase
													inpoint = 0 if inpoint < 0
													duration = (log[:stop_time] - log[:start_time]).round * timebase
													clipitem.add_child("<in>#{inpoint}</in>")
													clipitem.add_child("<out>#{inpoint+duration}</out>")
													clipitem.add_child("<start>#{playhead}</start>")
													clipitem.add_child("<end>#{playhead+duration}</end>")
													clipindex += 1
													clipitem.xpath("link/clipindex").each do |clipindex_node|
														clipindex_node.content = clipindex
													end
													xmeml << clipitem.to_s
													playhead += duration
												end
											end
										end
									}
								end
							}
						}
						xmeml.duration playhead
					}
				end # builder = 
				if playhead==2 * timebase
					@report << "No activities for Person: #{person}\n"
					@report << "--------------------------------------------\n"
				else
					if playhead < 0
						playhead = playhead.abs
						#puts "playhead #{playhead}"
					end
					if added_clips < 3
						@report << "Not enough activites for Person: #{person}\n"
						@report << "--------------------------------------------\n"
					else
						@report << "Creating string-out for #{person} activities\n"
						@report << "--------------------------------------------\n"
						sequence = builder.doc.at_xpath("sequence")
						children_node.add_child(sequence)
						added_stringouts += 1
					end
				end
			end # @people.each
		end # if @peopleActivitesCheckbox
		# for each person's keywords
		if @peopleKeywordsCheckbox.state==1
			@people.each_with_index do |person, i|
				#puts "#{i} #{person.inspect}"
				playhead, added_clips = 0, 0
				max_audio_tracks = 0
				builder = Nokogiri::XML::Builder.new do |xmeml|
					titles_clone = titles.dup
					xmeml.sequence(:id => "pk#{i+1}") {
						xmeml.name "#{person} keywords"
						xmeml.rate {
							xmeml.ntsc ntsc
							xmeml.timebase timebase
						}
						xmeml.timecode {
							xmeml.rate {
								xmeml.ntsc ntsc
								xmeml.timebase timebase
							}
							xmeml.frame "0"
							xmeml.displayformat displayformat
						}
						xmeml.in "-1"
						xmeml.out "-1"
						xmeml.media {
							xmeml.video {
								xmeml.format {
									xmeml.samplecharacteristics {
										xmeml.width width
										xmeml.height height
										xmeml.rate {
											xmeml.ntsc ntsc
											xmeml.timebase timebase
										}
										xmeml.colordepth "24"
									}
								}
								xmeml.track {
									clipindex = 0
									angle_index = -1
									stringout_clips.each do |clip|
										clip[:keywords].each do |log|
											#puts "log = #{log.inspect}"
											if log[:person]=="#{person} - Person"
												add_this_clip = true
												added_clips += 1
											else
												add_this_clip = false
											end
											#puts "add_this_clip? #{add_this_clip}"
											if add_this_clip
												audio_track_count = clip[:clip_node].xpath("media/audio/track/clipitem").length
												if audio_track_count > max_audio_tracks
													max_audio_tracks = audio_track_count
												end
												clipitem = clip[:clip_node].at_xpath("media/video/track/clipitem").dup
												angle_index += 1
												activevideoangle_nodes = clipitem.xpath("multiclip/angle/activevideoangle")
												if @multicamRadioButtons.selectedCell.tag==1 || angle_index==activevideoangle_nodes.length
													angle_index = 0
												end
												activevideoangle_nodes.each_with_index do |node, i|
													if i!=angle_index
														node.remove
													end
												end
												file_node = clipitem.at_xpath("file")
												file_node.children.each { |c| c.remove } # remove all children to leave a file reference
												in_node = clipitem.at_xpath("in")
												if in_node!= nil
													in_node.remove
												end
												out_node = clipitem.at_xpath("out")
												if out_node!=nil
													out_node.remove
												end
												inpoint = (log[:start_time] - clip[:start_time]).round * timebase
												inpoint = 0 if inpoint < 0
												duration = (log[:stop_time] - log[:start_time]).round * timebase
												clipitem.add_child("<in>#{inpoint}</in>")
												clipitem.add_child("<out>#{inpoint+duration}</out>")
												clipitem.add_child("<start>#{playhead}</start>")
												clipitem.add_child("<end>#{playhead+duration}</end>")
												clipindex += 1
												clipitem.xpath("link/clipindex").each do |clipindex_node|
													clipindex_node.content = clipindex
												end
												xmeml << clipitem.to_s
												playhead += duration
											end
										end
									end
								}
								xmeml.track { # titles
									playhead = 0
									stringout_clips.each do |clip|
										clip[:keywords].each do |log|
											#puts "log = #{log.inspect}"
											if log[:person]=="#{person} - Person"
												add_this_clip = true
												added_clips += 1
											else
												add_this_clip = false
											end
											#puts "add_this_clip? #{add_this_clip}"
											if add_this_clip
												inpoint = (log[:start_time] - clip[:start_time]).round * timebase
												inpoint = 0 if inpoint < 0
												duration = (log[:stop_time] - log[:start_time]).round * timebase
												if titles_clone.include?(log[:person]) && @titlesCheckbox.state==1
													titles_clone.delete(log[:person])
													xmeml << lower_third_xmeml(person, playhead, ntsc, timebase)
												end
												playhead += duration
											end
										end
									end
								}
							}
							xmeml.audio {
								xmeml.format {
									xmeml.samplecharacteristics {
										xmeml.depth "16"
										xmeml.samplerate samplerate
									}
								}
								max_audio_tracks.times do |i|
									xmeml.track {
										clipindex = 0
										playhead = 0 # reset for each track
										stringout_clips.each do |clip|
											clip[:keywords].each do |log|
												if log[:person]=="#{person} - Person"
													add_this_clip = true
												else
													add_this_clip = false
												end
												clipitems = clip[:clip_node].xpath("media/audio/track/clipitem")
												a_clipitem = clipitems[i-1]
												if a_clipitem.nil?
													add_this_clip = false
												end
												#puts "add_this_clip? #{add_this_clip}"
												if add_this_clip
													clipitem = a_clipitem.dup
													file_node = clipitem.at_xpath("file")
													file_node.children.each { |c| c.remove } # remove all children to leave a file reference
													in_node = clipitem.at_xpath("in")
													if in_node!= nil
														in_node.remove
													end
													out_node = clipitem.at_xpath("out")
													if out_node!=nil
														out_node.remove
													end
													inpoint = (log[:start_time] - clip[:start_time]).round * timebase
													inpoint = 0 if inpoint < 0
													duration = (log[:stop_time] - log[:start_time]).round * timebase
													clipitem.add_child("<in>#{inpoint}</in>")
													clipitem.add_child("<out>#{inpoint+duration}</out>")
													clipitem.add_child("<start>#{playhead}</start>")
													clipitem.add_child("<end>#{playhead+duration}</end>")
													clipindex += 1
													clipitem.xpath("link/clipindex").each do |clipindex_node|
														clipindex_node.content = clipindex
													end
													xmeml << clipitem.to_s
													playhead += duration
												end
											end
										end
									}
								end
							}
						}
						xmeml.duration playhead
					}
				end # builder = 
				if playhead==2 * timebase
					@report << "No keywords for Person: #{person}\n"
					@report << "--------------------------------------------\n"
				else
					if playhead < 0
						playhead = playhead.abs
						#puts "playhead #{playhead}"
					end
					if added_clips < 3
						@report << "Not enough keywords for Person: #{person}\n"
						@report << "--------------------------------------------\n"
					else
						@report << "Creating string-out for #{person} keywords\n"
						@report << "--------------------------------------------\n"
						sequence = builder.doc.at_xpath("sequence")
						children_node.add_child(sequence)
						added_stringouts += 1
					end
				end
			end # @people.each
		end # if @peopleKeywordsCheckbox
		# for each activity
		if @activitesCheckbox.state==1
			@activities.each_with_index do |activity, i|
				#puts "#{i}: #{activity.inspect}"
				playhead, added_clips = 0, 0
				max_audio_tracks = 0
				builder = Nokogiri::XML::Builder.new do |xmeml|
					titles_clone = titles.dup
					xmeml.sequence(:id => "ak#{i+1}") {
						xmeml.name "Activity: #{activity}"
						xmeml.rate {
							xmeml.ntsc ntsc
							xmeml.timebase timebase
						}
						xmeml.timecode {
							xmeml.rate {
								xmeml.ntsc ntsc
								xmeml.timebase timebase
							}
							xmeml.frame "0"
							xmeml.displayformat displayformat
						}
						xmeml.in "-1"
						xmeml.out "-1"
						xmeml.media {
							xmeml.video {
								xmeml.format {
									xmeml.samplecharacteristics {
										xmeml.width width
										xmeml.height height
										xmeml.rate {
											xmeml.ntsc ntsc
											xmeml.timebase timebase
										}
										xmeml.colordepth "24"
									}
								}
								xmeml.track {
									clipindex = 0
									angle_index = -1
									stringout_clips.each do |clip|
										clip[:activities].each do |log|
											#puts "log = #{log.inspect}"
											if log[:name]=="#{activity} - Activity"
												add_this_clip = true
												added_clips += 1
											else
												add_this_clip = false
											end
											#puts "add_this_clip? #{add_this_clip}"
											if add_this_clip
												audio_track_count = clip[:clip_node].xpath("media/audio/track/clipitem").length
												if audio_track_count > max_audio_tracks
													max_audio_tracks = audio_track_count
												end
												clipitem = clip[:clip_node].at_xpath("media/video/track/clipitem").dup
												angle_index += 1
												activevideoangle_nodes = clipitem.xpath("multiclip/angle/activevideoangle")
												if @multicamRadioButtons.selectedCell.tag==1 || angle_index==activevideoangle_nodes.length
													angle_index = 0
												end
												activevideoangle_nodes.each_with_index do |node, i|
													if i!=angle_index
														node.remove
													end
												end
												file_node = clipitem.at_xpath("file")
												file_node.children.each { |c| c.remove } # remove all children to leave a file reference
												in_node = clipitem.at_xpath("in")
												if in_node!= nil
													in_node.remove
												end
												out_node = clipitem.at_xpath("out")
												if out_node!=nil
													out_node.remove
												end
												inpoint = (log[:start_time] - clip[:start_time]).round * timebase
												#puts "inpoint = (#{log[:start_time]} - #{clip[:start_time]}).round * #{timebase} = #{inpoint}"
												inpoint = 0 if inpoint < 0
												duration = (log[:stop_time] - log[:start_time]).round * timebase
												#puts "duration = (#{log[:stop_time]} - #{log[:start_time]}).round * #{timebase} = #{duration}"
												clipitem.add_child("<in>#{inpoint}</in>")
												clipitem.add_child("<out>#{inpoint+duration}</out>")
												clipitem.add_child("<start>#{playhead}</start>")
												clipitem.add_child("<end>#{playhead+duration}</end>")
												clipindex += 1
												clipitem.xpath("link/clipindex").each do |clipindex_node|
													clipindex_node.content = clipindex
												end
												xmeml << clipitem.to_s
												playhead += duration
											end
										end
									end
								}
								xmeml.track { # titles
									playhead = 0
									stringout_clips.each do |clip|
										clip[:activities].each do |log|
											#puts "log = #{log.inspect}"
											if log[:name]=="#{activity} - Activity"
												add_this_clip = true
												added_clips += 1
											else
												add_this_clip = false
											end
											#puts "add_this_clip? #{add_this_clip}"
											if add_this_clip
												inpoint = (log[:start_time] - clip[:start_time]).round * timebase
												inpoint = 0 if inpoint < 0
												duration = (log[:stop_time] - log[:start_time]).round * timebase
												if titles_clone.include?(log[:person]) && @titlesCheckbox.state==1
													titles_clone.delete(log[:person])
													person = log[:person].sub(" - Person", "")
													xmeml << lower_third_xmeml(person, playhead, ntsc, timebase)
												end
												playhead += duration
											end
										end
									end
								}
							}
							xmeml.audio {
								xmeml.format {
									xmeml.samplecharacteristics {
										xmeml.depth "16"
										xmeml.samplerate samplerate
									}
								}
								max_audio_tracks.times do |i|
									xmeml.track {
										clipindex = 0
										playhead = 0 # reset for each track
										stringout_clips.each do |clip|
											clip[:activities].each do |log|
												if log[:name]=="#{activity} - Activity"
													add_this_clip = true
												else
													add_this_clip = false
												end
												clipitems = clip[:clip_node].xpath("media/audio/track/clipitem")
												a_clipitem = clipitems[i-1]
												if a_clipitem.nil?
													add_this_clip = false
												end
												#puts "add_this_clip? #{add_this_clip}"
												if add_this_clip
													clipitem = a_clipitem.dup
													file_node = clipitem.at_xpath("file")
													file_node.children.each { |c| c.remove } # remove all children to leave a file reference
													in_node = clipitem.at_xpath("in")
													if in_node!= nil
														in_node.remove
													end
													out_node = clipitem.at_xpath("out")
													if out_node!=nil
														out_node.remove
													end
													inpoint = (log[:start_time] - clip[:start_time]).round * timebase
													inpoint = 0 if inpoint < 0
													duration = (log[:stop_time] - log[:start_time]).round * timebase
													clipitem.add_child("<in>#{inpoint}</in>")
													clipitem.add_child("<out>#{inpoint+duration}</out>")
													clipitem.add_child("<start>#{playhead}</start>")
													clipitem.add_child("<end>#{playhead+duration}</end>")
													clipindex += 1
													clipitem.xpath("link/clipindex").each do |clipindex_node|
														clipindex_node.content = clipindex
													end
													xmeml << clipitem.to_s
													playhead += duration
												end
											end
										end
									}
								end
							}
						}
						xmeml.duration playhead
					}
				end # builder = 
				if playhead==2 * timebase
					@report << "No media for Activity: #{activity}\n"
					@report << "--------------------------------------------\n"
				else
					if playhead < 0
						playhead = playhead.abs
						#puts "playhead #{playhead}"
					end
					if added_clips < 3
						@report << "Not enough media for Activity: #{activity}\n"
						@report << "--------------------------------------------\n"
					else
						@report << "Creating string-out for Activity: #{activity}\n"
						@report << "--------------------------------------------\n"
						sequence = builder.doc.at_xpath("sequence")
						children_node.add_child(sequence)
						added_stringouts += 1
					end
				end
			end # @activities.each
		end # if @activitesCheckbox
		# for each other keyword
		if @keywordsCheckbox.state==1
			@keywords.each_with_index do |keyword, i|
				#puts "#{i} #{keyword.inspect}"
				playhead, added_clips = 0, 0
				max_audio_tracks = 0
				builder = Nokogiri::XML::Builder.new do |xmeml|
					titles_clone = titles.dup
					xmeml.sequence(:id => "ok#{i+1}") {
						xmeml.name "Keyword: #{keyword}"
						xmeml.rate {
							xmeml.ntsc ntsc
							xmeml.timebase timebase
						}
						xmeml.timecode {
							xmeml.rate {
								xmeml.ntsc ntsc
								xmeml.timebase timebase
							}
							xmeml.frame "0"
							xmeml.displayformat displayformat
						}
						xmeml.in "-1"
						xmeml.out "-1"
						xmeml.media {
							xmeml.video {
								xmeml.format {
									xmeml.samplecharacteristics {
										xmeml.width width
										xmeml.height height
										xmeml.rate {
											xmeml.ntsc ntsc
											xmeml.timebase timebase
										}
										xmeml.colordepth "24"
									}
								}
								xmeml.track {
									clipindex = 0
									angle_index = -1
									stringout_clips.each do |clip|
										clip[:keywords].each do |log|
											#puts "log = #{log.inspect}"
											if log[:name]=="#{keyword} - Other"
												add_this_clip = true
												added_clips += 1
											else
												add_this_clip = false
											end
											#puts "add_this_clip? #{add_this_clip}"
											if add_this_clip
												audio_track_count = clip[:clip_node].xpath("media/audio/track/clipitem").length
												if audio_track_count > max_audio_tracks
													max_audio_tracks = audio_track_count
												end
												clipitem = clip[:clip_node].at_xpath("media/video/track/clipitem").dup
												angle_index += 1
												activevideoangle_nodes = clipitem.xpath("multiclip/angle/activevideoangle")
												if @multicamRadioButtons.selectedCell.tag==1 || angle_index==activevideoangle_nodes.length
													angle_index = 0
												end
												activevideoangle_nodes.each_with_index do |node, i|
													if i!=angle_index
														node.remove
													end
												end
												file_node = clipitem.at_xpath("file")
												file_node.children.each { |c| c.remove } # remove all children to leave a file reference
												in_node = clipitem.at_xpath("in")
												if in_node!= nil
													in_node.remove
												end
												out_node = clipitem.at_xpath("out")
												if out_node!=nil
													out_node.remove
												end
												inpoint = (log[:start_time] - clip[:start_time]).round * timebase
												inpoint = 0 if inpoint < 0
												duration = (log[:stop_time] - log[:start_time]).round * timebase
												clipitem.add_child("<in>#{inpoint}</in>")
												clipitem.add_child("<out>#{inpoint+duration}</out>")
												clipitem.add_child("<start>#{playhead}</start>")
												clipitem.add_child("<end>#{playhead+duration}</end>")
												clipindex += 1
												clipitem.xpath("link/clipindex").each do |clipindex_node|
													clipindex_node.content = clipindex
												end
												xmeml << clipitem.to_s
												playhead += duration
											end
										end
									end
								}
								xmeml.track { # titles
									playhead = 0
									stringout_clips.each do |clip|
										clip[:keywords].each do |log|
											#puts "log = #{log.inspect}"
											if log[:name]=="#{keyword} - Other"
												add_this_clip = true
												added_clips += 1
											else
												add_this_clip = false
											end
											#puts "add_this_clip? #{add_this_clip}"
											if add_this_clip
												inpoint = (log[:start_time] - clip[:start_time]).round * timebase
												inpoint = 0 if inpoint < 0
												duration = (log[:stop_time] - log[:start_time]).round * timebase
												if titles_clone.include?(log[:person]) && @titlesCheckbox.state==1
													titles_clone.delete(log[:person])
													person = log[:person].sub(" - Person", "")
													xmeml << lower_third_xmeml(person, playhead, ntsc, timebase)
												end
												playhead += duration
											end
										end
									end
								}
							}
							xmeml.audio {
								xmeml.format {
									xmeml.samplecharacteristics {
										xmeml.depth "16"
										xmeml.samplerate samplerate
									}
								}
								max_audio_tracks.times do |i|
									xmeml.track {
										clipindex = 0
										playhead = 0 # reset for each track
										stringout_clips.each do |clip|
											clip[:keywords].each do |log|
												#puts "log = #{log.inspect}"
												if log[:name]=="#{keyword} - Other"
													add_this_clip = true
												else
													add_this_clip = false
												end
												clipitems = clip[:clip_node].xpath("media/audio/track/clipitem")
												a_clipitem = clipitems[i-1]
												if a_clipitem.nil?
													add_this_clip = false
												end
												#puts "add_this_clip? #{add_this_clip}"
												if add_this_clip
													clipitem = a_clipitem.dup
													file_node = clipitem.at_xpath("file")
													file_node.children.each { |c| c.remove } # remove all children to leave a file reference
													in_node = clipitem.at_xpath("in")
													if in_node!= nil
														in_node.remove
													end
													out_node = clipitem.at_xpath("out")
													if out_node!=nil
														out_node.remove
													end
													inpoint = (log[:start_time] - clip[:start_time]).round * timebase
													inpoint = 0 if inpoint < 0
													duration = (log[:stop_time] - log[:start_time]).round * timebase
													clipitem.add_child("<in>#{inpoint}</in>")
													clipitem.add_child("<out>#{inpoint+duration}</out>")
													clipitem.add_child("<start>#{playhead}</start>")
													clipitem.add_child("<end>#{playhead+duration}</end>")
													clipindex += 1
													clipitem.xpath("link/clipindex").each do |clipindex_node|
														clipindex_node.content = clipindex
													end
													xmeml << clipitem.to_s
													playhead += duration
												end
											end
										end
									}
								end
							}
						}
						xmeml.duration playhead
					}
				end # builder = 
				if playhead==2 * timebase
					@report << "No activities for Keyword: #{keyword}\n"
					@report << "--------------------------------------------\n"
				else
					if playhead < 0
						playhead = playhead.abs
						#puts "playhead #{playhead}"
					end
					if added_clips < 3
						@report << "Not enough media for Keyword: #{keyword}\n"
						@report << "--------------------------------------------\n"
					else
						@report << "Creating string-out for Keyword: #{keyword}\n"
						@report << "--------------------------------------------\n"
						sequence = builder.doc.at_xpath("sequence")
						children_node.add_child(sequence)
						added_stringouts += 1
					end
				end
			end # @keywords.each
		end # if @keywordsCheckbox
		# for each favorite
		if @favoritesCheckbox.state==1
			playhead, added_clips = 0, 0
			max_audio_tracks = 0
			builder = Nokogiri::XML::Builder.new do |xmeml|
				titles_clone = titles.dup
				xmeml.sequence(:id => "fav1") {
					xmeml.name "Favorites"
					xmeml.rate {
						xmeml.ntsc ntsc
						xmeml.timebase timebase
					}
					xmeml.timecode {
						xmeml.rate {
							xmeml.ntsc ntsc
							xmeml.timebase timebase
						}
						xmeml.frame "0"
						xmeml.displayformat displayformat
					}
					xmeml.in "-1"
					xmeml.out "-1"
					xmeml.media {
						xmeml.video {
							xmeml.format {
								xmeml.samplecharacteristics {
									xmeml.width width
									xmeml.height height
									xmeml.rate {
										xmeml.ntsc ntsc
										xmeml.timebase timebase
									}
									xmeml.colordepth "24"
								}
							}
							xmeml.track {
								clipindex = 0
								angle_index = -1
								stringout_clips.each do |clip|
									clip[:favorites].each do |log|
										#puts "log = #{log.inspect}"
										add_this_clip = true
										added_clips += 1
										audio_track_count = clip[:clip_node].xpath("media/audio/track/clipitem").length
										if audio_track_count > max_audio_tracks
											max_audio_tracks = audio_track_count
										end
										clipitem = clip[:clip_node].at_xpath("media/video/track/clipitem").dup
										angle_index += 1
										activevideoangle_nodes = clipitem.xpath("multiclip/angle/activevideoangle")
										if @multicamRadioButtons.selectedCell.tag==1 || angle_index==activevideoangle_nodes.length
											angle_index = 0
										end
										activevideoangle_nodes.each_with_index do |node, i|
											if i!=angle_index
												node.remove
											end
										end
										file_node = clipitem.at_xpath("file")
										file_node.children.each { |c| c.remove } # remove all children to leave a file reference
										in_node = clipitem.at_xpath("in")
										if in_node!= nil
											in_node.remove
										end
										out_node = clipitem.at_xpath("out")
										if out_node!=nil
											out_node.remove
										end
										inpoint = (log[:start_time] - clip[:start_time]).round * timebase
										inpoint = 0 if inpoint < 0
										duration = (log[:stop_time] - log[:start_time]).round * timebase
										clipitem.add_child("<in>#{inpoint}</in>")
										clipitem.add_child("<out>#{inpoint+duration}</out>")
										clipitem.add_child("<start>#{playhead}</start>")
										clipitem.add_child("<end>#{playhead+duration}</end>")
										clipindex += 1
										clipitem.xpath("link/clipindex").each do |clipindex_node|
											clipindex_node.content = clipindex
										end
										xmeml << clipitem.to_s
										playhead += duration
									end
								end
							}
							xmeml.track { # titles
								playhead = 0
								stringout_clips.each do |clip|
									clip[:favorites].each do |log|
										inpoint = (log[:start_time] - clip[:start_time]).round * timebase
										inpoint = 0 if inpoint < 0
										duration = (log[:stop_time] - log[:start_time]).round * timebase
										if titles_clone.include?(log[:person]) && @titlesCheckbox.state==1
											titles_clone.delete(log[:person])
											person = log[:person].sub(" - Person", "")
											xmeml << lower_third_xmeml(person, playhead, ntsc, timebase)
										end
										playhead += duration
									end
								end
							}
						}
						xmeml.audio {
							xmeml.format {
								xmeml.samplecharacteristics {
									xmeml.depth "16"
									xmeml.samplerate samplerate
								}
							}
							max_audio_tracks.times do |i|
								xmeml.track {
									clipindex = 0
									playhead = 0 # reset for each track
									stringout_clips.each do |clip|
										clip[:favorites].each do |log|
											add_this_clip = true
											clipitems = clip[:clip_node].xpath("media/audio/track/clipitem")
											a_clipitem = clipitems[i-1]
											if a_clipitem.nil?
												add_this_clip = false
											end
											#puts "FAVORITES AUDIO add_this_clip? #{add_this_clip}"
											if add_this_clip
												clipitem = a_clipitem.dup
												file_node = clipitem.at_xpath("file")
												file_node.children.each { |c| c.remove } # remove all children to leave a file reference
												in_node = clipitem.at_xpath("in")
												if in_node!= nil
													in_node.remove
												end
												out_node = clipitem.at_xpath("out")
												if out_node!=nil
													out_node.remove
												end
												inpoint = (log[:start_time] - clip[:start_time]).round * timebase
												inpoint = 0 if inpoint < 0
												duration = (log[:stop_time] - log[:start_time]).round * timebase
												clipitem.add_child("<in>#{inpoint}</in>")
												clipitem.add_child("<out>#{inpoint+duration}</out>")
												clipitem.add_child("<start>#{playhead}</start>")
												clipitem.add_child("<end>#{playhead+duration}</end>")
												clipindex += 1
												clipitem.xpath("link/clipindex").each do |clipindex_node|
													clipindex_node.content = clipindex
												end
												xmeml << clipitem.to_s
												playhead += duration
											end
										end
									end
								}
							end
						}
					}
					xmeml.duration playhead
				}
			end # builder = 
			if playhead==2 * timebase
				@report << "No media for Favorites\n"
				@report << "--------------------------------------------\n"
			else
				if playhead < 0
					playhead = playhead.abs
					#puts "playhead #{playhead}"
				end
				if added_clips < 3
					@report << "Not enough media for Favorites\n"
					@report << "--------------------------------------------\n"
				else
					@report << "Creating string-out for Favorites\n"
					@report << "--------------------------------------------\n"
					sequence = builder.doc.at_xpath("sequence")
					children_node.add_child(sequence)
					added_stringouts += 1
				end
			end
		end # if @favoritesCheckbox
		return added_stringouts
	end
	
	def lower_third_xmeml(person, playhead, ntsc, timebase)
		one_second = timebase
		lower_third_str = "
		<transitionitem>
		<start>#{playhead}</start>
		<end>#{playhead + one_second}</end>
		<alignment>start-black</alignment>
		<rate>
		<timebase>#{timebase}</timebase>
		<ntsc>#{ntsc}</ntsc>
		</rate>
		<effect>
		<name>Cross Dissolve</name>
		<effectid>Cross Dissolve</effectid>
		<effectcategory>Dissolve</effectcategory>
		<effecttype>transition</effecttype>
		<mediatype>video</mediatype>
		<wipecode>0</wipecode>
		<wipeaccuracy>100</wipeaccuracy>
		<startratio>0</startratio>
		<endratio>1</endratio>
		<reverse>FALSE</reverse>
		</effect>
		</transitionitem>
		<generatoritem id='Lower 3rd'>
		<name>Lower 3rd</name>
		<duration>3000</duration>
		<rate>
		<timebase>#{timebase}</timebase>
		<ntsc>#{ntsc}</ntsc>
		</rate>
		<start>-1</start>
		<end>-1</end>
		<alphatype>black</alphatype>
		<effect>
		<name>Lower 3rd</name>
		<effectid>Lower 3rd</effectid>
		<effectcategory>Text</effectcategory>
		<effecttype>generator</effecttype>
		<mediatype>video</mediatype>
		<parameter>
		<parameterid>str2</parameterid>
		<name>Text 2</name>
		<value>#{person}</value>
		</parameter>
		<parameter>
		<parameterid>font2</parameterid>
		<name>Font</name>
		<value>Helvetica</value>
		</parameter>
		<parameter>
		<parameterid>style2</parameterid>
		<name>Style</name>
		<valuemin>1</valuemin>
		<valuemax>4</valuemax>
		<valuelist>
		<valueentry>
		<name>Plain</name>
		<value>1</value>
		</valueentry>
		<valueentry>
		<name>Bold</name>
		<value>2</value>
		</valueentry>
		<valueentry>
		<name>Italic</name>
		<value>3</value>
		</valueentry>
		<valueentry>
		<name>Bold/Italic</name>
		<value>4</value>
		</valueentry>
		</valuelist>
		<value>1</value>
		</parameter>
		<parameter>
		<parameterid>size2</parameterid>
		<name>Size</name>
		<valuemin>4</valuemin>
		<valuemax>200</valuemax>
		<value>36</value>
		</parameter>
		<parameter>
		<parameterid>track2</parameterid>
		<name>Tracking</name>
		<valuemin>-200</valuemin>
		<valuemax>200</valuemax>
		<value>1</value>
		</parameter>
		<parameter>
		<parameterid>color2</parameterid>
		<name>Font Color</name>
		<value>
		<alpha>255</alpha>
		<red>255</red>
		<green>255</green>
		<blue>255</blue>
		</value>
		</parameter>
		<parameter>
		<parameterid>bgtype</parameterid>
		<name>Background</name>
		<valuemin>1</valuemin>
		<valuemax>3</valuemax>
		<valuelist>
		<valueentry>
		<name>None</name>
		<value>1</value>
		</valueentry>
		<valueentry>
		<name>Bar</name>
		<value>2</value>
		</valueentry>
		<valueentry>
		<name>Solid</name>
		<value>3</value>
		</valueentry>
		</valuelist>
		<value>1</value>
		</parameter>
		<parameter>
		<parameterid>bgopacity</parameterid>
		<name>Opacity</name>
		<valuemin>0</valuemin>
		<valuemax>100</valuemax>
		<value>100</value>
		</parameter>
		<parameter>
		<parameterid>bgcolor</parameterid>
		<name>Color</name>
		<value>
		<alpha>255</alpha>
		<red>0</red>
		<green>0</green>
		<blue>0</blue>
		</value>
		</parameter>
		<parameter>
		<parameterid>autokern</parameterid>
		<name>Auto Kerning</name>
		<value>TRUE</value>
		</parameter>
		</effect>
		<sourcetrack>
		<mediatype>video</mediatype>
		</sourcetrack>
		</generatoritem>
		<transitionitem>
		<start>#{playhead + 3*one_second}</start>
		<end>#{playhead + 4*one_second}</end>
		<alignment>end-black</alignment>
		<rate>
		<timebase>#{timebase}</timebase>
		<ntsc>#{ntsc}</ntsc>
		</rate>
		<effect>
		<name>Cross Dissolve</name>
		<effectid>Cross Dissolve</effectid>
		<effectcategory>Dissolve</effectcategory>
		<effecttype>transition</effecttype>
		<mediatype>video</mediatype>
		<wipecode>0</wipecode>
		<wipeaccuracy>100</wipeaccuracy>
		<startratio>0</startratio>
		<endratio>1</endratio>
		<reverse>FALSE</reverse>
		</effect>
		</transitionitem>
		"
		return lower_third_str
	end
	
	def applicationShouldTerminateAfterLastWindowClosed(sender)
		true
	end
	
	def applicationWillTerminate(sender)
		login(self)
	end
	
	### TableView methods ###

	def numberOfRowsInTableView(tableView)
		if tableView.tag==1 # Keywords
			if @metadata_logs.nil?
				return 0
			else
				return @metadata_logs.length
			end
		elsif tableView.tag==2 # Media
			if @clips.nil?
				return 0
			else
				return @clips.length
			end
		end
	end

	def tableView_objectValueForTableColumn_row(tableView, tableColumn, rowIndex)
		if tableView.tag==1 && @metadata_logs!=nil && @metadata_logs.length>0 # Keywords
			object = @metadata_logs[rowIndex]
			case tableColumn.headerCell.stringValue
			when "Logged Keywords"
				return object["metadata"] unless object.nil?
			when "Transcript"
				unless object.nil?
					if object[:relevance].nil? # Transcript keyword
						return object["metadata"]
					else # Magic keyword with checkbox
						return object[:checkbox]
					end
				end
			when "Start Time"
				return object["local_start_time"] unless object.nil?
			when "Stop Time"
				return object["local_stop_time"] unless object.nil?
			when "Start Timecode"
				return object["start_timecode"] unless object.nil?
			when "Stop Timecode"
				return object["stop_timecode"] unless object.nil?
			end
		elsif tableView.tag==2 && @clips!=nil && @clips.length>0 # Media
			object = @clips[rowIndex]
			case tableColumn.headerCell.stringValue
			when "Media Files", "Clips"
				return object["clip_name"] unless object.nil?
			when "Start Time"
				return object["local_start_time"] unless object.nil?
			when "Stop Time"
				return object["local_stop_time"] unless object.nil?
			when "Start Timecode"
				return object["start_timecode"] unless object.nil?
			when "Stop Timecode"
				return object["stop_timecode"] unless object.nil?
			end
		end
	end
	
	# default cell in nib is Text Field Cell
	def tableView_dataCellForTableColumn_row(tableView, tableColumn, rowIndex)
		if tableColumn!=nil && tableColumn.headerCell.stringValue=="Transcript"
			object = @metadata_logs[rowIndex]
			if object[:relevance].nil?
				# puts "Transcript keyword for #{object[:speaker]}"
			else
				# puts "Magic keyword #{object[:name]} for #{object[:speaker]}"
				@checkBoxCell.setTitle(object[:name])
				return @checkBoxCell # override default
			end
		end
	end

	def tableView_shouldTrackCell_forTableColumn_row(tableView, cell, tableColumn, rowIndex) # triggered by clicking on the row
		tableView.deselectRow(rowIndex)
		if tableColumn!=nil && tableColumn.headerCell.stringValue=="Transcript"
			object = @metadata_logs[rowIndex]
			if object[:relevance].nil?
				#puts "Transcript keyword for #{object[:speaker]}"
			else
				#puts "Magic keyword #{object[:name]} for #{object[:speaker]}"
				same_magic_keyword = @extracted_logs.find_all { |e| e[:name]==object[:name] }
				if object[:checkbox]==0
					same_magic_keyword.each { |k| k[:checkbox] = 1 }
				else
					same_magic_keyword.each { |k| k[:checkbox] = 0 }
				end
				filterMagicKeywordsByRelevance(self)
			end
		end
		return true
	end
		
	# def tableView_setObjectValue_forTableColumn_row(tableView, anObject, tableColumn, rowIndex)
	# 	#puts "tableView_setObjectValue_forTableColumn_row"
	# end

end
